/*****************************************************************************
 * id3lib - a library for manipulating id3 tags                              *
 * Copyright (C) 2000 Scott Guilbeaux <scottg@viptx.net>                     *
 *                                                                           *
 * This library is free software; you can redistribute it and/or modify      *
 * it under the terms of the GNU General Public License as published by      *
 * the Free Software Foundation; either version 2 of the License, or         *
 * (at your option) any later version.                                       *
 *                                                                           *
 * This library is distributed in the hope that it will be useful,           *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of            *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
 * GNU General Public License for more details.                              *
 *                                                                           *
 * You should have received a copy of the GNU General Public License         *
 * along with this library; if not, write to the Free Software               *
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA *
 *****************************************************************************/

#include "id3.h"

/* global file name pointer */
static char *selected_mp3_file;

/* initialize id3 tag */
int id3_init(id3tag_t *tag)
{    
   strcpy(tag->tag, "TAG");
   tag->title[0] = '\0';
   tag->artist[0] = '\0';
   tag->album[0] = '\0';
   tag->comment[0] = '\0';
   tag->year[0] = '\0';
   tag->genre = 0xFF;
   return(1);
}    

/* sort our array of genres */
void id3_sort_genres(void)
{
   int i, j;
   char *stmp;
   int itmp;
   
   if(id3_genres_sorted[0].name != NULL)
     return;
   
   for(i = 0; i < GENRE_MAX; i++)
     id3_genres_sorted[i] = id3_genres[i];
   
   for(j = GENRE_MAX - 2; j > 0; j--)
     for(i = 0; i < j; i++)
     {
	if(strcmp(id3_genres_sorted[i].name, id3_genres_sorted[i+1].name) > 0)
	  {
	     stmp = id3_genres_sorted[i].name;
	     itmp = id3_genres_sorted[i].id;
	     id3_genres_sorted[i] = id3_genres_sorted[i+1];
	     id3_genres_sorted[i+1].name = stmp;
	     id3_genres_sorted[i+1].id = itmp;
	  }
     }
   id3_genres_sorted[GENRE_MAX].name = "";
}

/* start an id3 session for reading */
int id3_init_read_session(char *filename)
{
   selected_mp3_file = (char *)malloc(strlen(filename) + 1);
   strcpy(selected_mp3_file, filename);
   mp3file = fopen(filename, "r");
   if(mp3file == NULL)
     return 0;
   return 1;
};

/* start an id3 session for writing */
int id3_init_write_session(char *filename)
{
   selected_mp3_file = (char *)malloc(strlen(filename) + 1);
   strcpy(selected_mp3_file, filename);
   mp3file = fopen(filename, "r+");
   if(mp3file == NULL)
     return 0;
   return 1;
}

/* close our id3 session */
void id3_close_session(void)
{
   fclose(mp3file);
   free(selected_mp3_file);
}

/* read tag from file and return
 * our id3 struct */
id3tag_t id3_read_tag(void)
{
   id3tag_t tag;
   
   fseek(mp3file, -128, SEEK_END);
   
   fread(&(tag.tag), 1, 3, mp3file);
   if(strncmp(tag.tag, "TAG", 3) != 0)
     return(tag);
   
   fread(&(tag.title), 1, 30, mp3file); 
   tag.title[30] = '\0';
   fread(&(tag.artist), 1, 30, mp3file);
   tag.artist[30] = '\0';
   fread(&(tag.album), 1, 30, mp3file);
   tag.album[30] = '\0';
   fread(&(tag.year), 1, 4, mp3file);
   tag.year[4] = '\0';
   fread(&(tag.comment), 1, 30, mp3file);
   tag.comment[30] = '\0';
   fread(&(tag.genre), 1, 1, mp3file);
   
   id3_field_trim(tag.title);
   id3_field_trim(tag.artist);
   id3_field_trim(tag.album);
   id3_field_trim(tag.year);
   id3_field_trim(tag.comment);
   
   return(tag);
}

/* appends a new empty tag to
 * our file */
int id3_new_tag(void)
{
   FILE *fp;
   id3tag_t tag;
   
   fp = fopen(selected_mp3_file, "a");
   if(fp)
     {
	strcpy(tag.tag, "TAG");
	strcpy(tag.title, "                             " );
	strcpy(tag.artist, "                             " );
	strcpy(tag.album, "                             " );
	strcpy(tag.comment, "                             " );
	strcpy(tag.year, "    " );
	tag.genre=0xFF;
	
	fseek(fp, 0, SEEK_END);
	fwrite((tag.tag), 1, 3, fp);
	fwrite((tag.title), 1, 30, fp);
	fwrite((tag.artist), 1, 30, fp);
	fwrite((tag.album), 1, 30, fp);
	fwrite((tag.comment), 1, 30, fp);
	fwrite((tag.year), 1, 4, fp);
	fwrite(&(tag.genre), 1, 1, fp);
	
	fclose(fp);
     }
   else
     return(-1);
   return 1;
}

/* set a new id3 tag to our file */
int id3_set_tag(id3tag_t tag)
{
   char buffer[3];
   
   /* If the file has no id3 tag
    * then create one */
   fseek(mp3file, -128, SEEK_END);
   fread(&(buffer), 1, 3, mp3file);

   if(strncmp(buffer, "TAG", 3) != 0)
     {
	fclose(mp3file);
	id3_new_tag();
	mp3file = fopen(selected_mp3_file, "r+");
	if(!mp3file)
	  return(-1);
     }
   fseek(mp3file, -128, SEEK_END);
   strcpy(tag.tag, "TAG");
   id3_fill_field(tag.tag, 3);
   fwrite((tag.tag), 1, 3, mp3file);
   
   if(tag.title[0] != '\0')
     {
	fseek(mp3file, -125, SEEK_END);
	id3_fill_field(tag.title, 30);
	fwrite((tag.title), 1, 30, mp3file);
     }
   if(tag.artist[0] != '\0')
     {
	fseek(mp3file, -95, SEEK_END);
	id3_fill_field(tag.artist, 30);
	fwrite((tag.artist), 1, 30, mp3file);
     }
   if(tag.album[0] != '\0')
     {
	fseek(mp3file, -65, SEEK_END);
	id3_fill_field(tag.album, 30);
	fwrite((tag.album), 1, 30, mp3file);
     }
   if(tag.year[0] != '\0')
     {
	fseek(mp3file, -35, SEEK_END);
	id3_fill_field(tag.year, 4);
	fwrite((tag.year), 1, 4, mp3file);
     }
   if(tag.comment[0] != '\0')
     {
	fseek(mp3file, -31, SEEK_END);
	id3_fill_field(tag.comment,30);
	fwrite((tag.comment), 1, 30, mp3file);
     }
   
   if(tag.genre != 0xFF)
     {
	fseek(mp3file, -1, SEEK_END);
	fwrite(&(tag.genre), 1, 1, mp3file);
     }
   else
     {
	tag.genre = 0xFF;
	fseek(mp3file, -1, SEEK_END);
	fwrite(&(tag.genre), 1, 1, mp3file);
     }
   return 1;
}

/* pad our field with extra spaces */
void id3_fill_field(char *str, int end)
{
   int i = 0;
   
   for(i = strlen(str); i < end; i++)
     str[i] = ' ';
   str[end] = '\0';
}

/* remove extra spaces from our field */
void id3_field_trim(char *buffer)
{
   int i = 0;
   
   for(i = strlen(buffer); !(isgraph(buffer[i])) && !(buffer[i] & 0x80); i--)
     if(i < 0)
       break;
   buffer[++i] = '\0';
}

/* returns the genre id when given the name */
int id3_get_genre_id_by_name(char *name)
{
   int i = 0;
   
   for(i = 0; id3_genres[i].name != NULL; i++)
     {
	if((strncmp(id3_genres[i].name, name, strlen(name)) == 0))
	  return(id3_genres[i].id);
     }
   return(0xFF);
}

/* returns the genre name when given the id */
char *id3_find_genre_by_id(int genre)
{
   int ctr;
   for(ctr = 0; id3_genres[ctr].name != NULL; ctr++)
     {
	if(id3_genres[ctr].id == genre)
	  return(id3_genres[ctr].name);
     }
   return(NULL);
}

/* return 1 if tag exists else return 0 */

int id3_check_for_tag(char *filename)
{
   FILE *fp;
   id3tag_t tag;
   int retval = 0;
   
   fp = fopen(filename, "r");
   if(fp == NULL)
     return -1;
   
   fseek(fp, -128, SEEK_END);
   fread(&(tag.tag), 1, 3, fp);
   if(strncmp(tag.tag, "TAG", 3) == 0)
     retval = 1;
   else
     retval = 0;
   
   return retval;
}
