/*****************************************************************************
 * PDMP3 - A mp3 manager for GNOME                                           *
 * Copyright (C) 2000 Scott Guilbeaux <scottg@viptx.net>                     *
 *                                                                           *
 * This program is free software; you can redistribute it and/or modify      *
 * it under the terms of the GNU General Public License as published by      *
 * the Free Software Foundation; either version 2 of the License, or         *
 * (at your option) any later version.                                       *
 *                                                                           *
 * This program is distributed in the hope that it will be useful,           *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of            *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
 * GNU General Public License for more details.                              *
 *                                                                           *
 * You should have received a copy of the GNU General Public License         *
 * along with this program; if not, write to the Free Software               *
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA *
 *****************************************************************************/

#include "pdmp3.h"

/* Program exit callback */

void pdmp3_quit_cb(GtkWidget *widget, GdkEvent *event, gpointer data)
{
   pdmp3_gui_shutdown();
   
   gtk_main_quit();

   pdmp3_shutdown();
}

/* Program about callback */

void pdmp3_about_cb(GtkWidget *widget, GdkEvent *event, gpointer data)
{
   static GtkWidget *dialog = NULL;
   const gchar *authors[] = { "Scott Guilbeaux <scottg@viptx.net>", NULL };
 
   if(dialog == NULL)
     {
	dialog = gnome_about_new(PACKAGE, VERSION, COPYRIGHT, authors,
				 DESCRIPTION, "pdmp3-about.xpm");
	gtk_signal_connect(GTK_OBJECT(dialog), "destroy",
			   GTK_SIGNAL_FUNC(gtk_widget_destroyed),
			   &dialog);
	gnome_dialog_set_parent(GNOME_DIALOG(dialog), GTK_WINDOW(pdmp3.main_window));
	gtk_widget_show(dialog);
     }
}

/* Session save callback */
gint pdmp3_save_session_cb(GnomeClient *client, gint phase, GnomeSaveStyle save_style,
			   gint is_shutdown, GnomeInteractStyle interact_style,
			   gint is_fast, gpointer client_data)
{
   gchar **argv;
   guint argc;
   
   argv = g_malloc0(sizeof(gchar *)*4);
   argc = 1;
   
   argv[0] = (gchar *)client_data;
   gnome_client_set_clone_command(client, argc, argv);
   gnome_client_set_restart_command(client, argc, argv);
   
   return TRUE;
}

/* Session die callback */

void pdmp3_session_die_cb(GnomeClient *client, gpointer client_data)
{
   pdmp3_gui_shutdown();
   
   gtk_main_quit();
   
   pdmp3_shutdown();
}

/*
 * Callback for when a row is selected
 * in our file list
 */

void pdmp3_file_list_row_selected_cb(GtkWidget *widget, gint row, gint column, GdkEventButton *bevent, gpointer data)
{
   gpointer ptr = NULL;
   gchar *filename;
   gchar *fullpath;
   id3tag_t mytag;
   
   pdmp3.selected_file_list_row = row;
   
   ptr = gtk_clist_get_row_data(GTK_CLIST(pdmp3.file_list), row);
   gtk_clist_get_text(GTK_CLIST(pdmp3.file_list), row, 1, &filename);
   fullpath = g_strdup_printf("%s/%s", (gchar *)ptr, filename);
   pdmp3.selected_file = g_strdup_printf("%s", fullpath);
   pdmp3_update_menubar();
   pdmp3_update_toolbar();
   gnome_appbar_set_status(GNOME_APPBAR(pdmp3.statusbar), filename);
   
   mytag = pdmp3_read_id3_tag();
   
   pdmp3_display_id3_tag(mytag);
}

/*
 * Callback for when a mouse button is pressed
 * in our file list
 */

void pdmp3_file_list_button_pressed_cb(GtkWidget *widget, GdkEventButton *bevent)
{
   int row = 0;
   int col = 0;
   
   if(bevent->button == 3)
     {
	gtk_clist_get_selection_info(GTK_CLIST(pdmp3.file_list),
				     bevent->x, bevent->y, &row, &col);
	if(row != pdmp3.selected_file_list_row)
	  gtk_clist_select_row(GTK_CLIST(pdmp3.file_list), row, 0);
	
	/* Disable or enable menu items depending on 
	 * program state */
	if(pdmp3.selected_file)
	  {
	     gtk_widget_set_sensitive(file_list_file_menu[0].widget, TRUE);
	     gtk_widget_set_sensitive(file_list_file_menu[1].widget, TRUE);
	     gtk_widget_set_sensitive(file_list_file_menu[2].widget, TRUE);
	     gtk_widget_set_sensitive(file_list_edit_menu[0].widget, TRUE);
	     gtk_widget_set_sensitive(file_list_edit_menu[2].widget, TRUE);
	  }
	else
	  {
	     gtk_widget_set_sensitive(file_list_file_menu[0].widget, FALSE);
	     gtk_widget_set_sensitive(file_list_file_menu[1].widget, FALSE);
	     gtk_widget_set_sensitive(file_list_file_menu[2].widget, FALSE);
	     gtk_widget_set_sensitive(file_list_edit_menu[0].widget, FALSE);
	     gtk_widget_set_sensitive(file_list_edit_menu[2].widget, FALSE);
	  }
	
	gtk_menu_popup(GTK_MENU(pdmp3.file_list_menu), NULL, NULL, NULL,
		       NULL, bevent->button, bevent->time);
     }
}

/*
 * Callback for when a button is pressed 
 * in our directory browser
 */

void pdmp3_dir_browser_button_pressed_cb(GtkWidget *widget, GdkEventButton *bevent)
{
   int row = 0;
   int col = 0;
   
   if(bevent->button == 3)
     {
	GList* l = NULL;
	gtk_clist_get_selection_info(GTK_CLIST(pdmp3.dir_browser),
				     bevent->x, bevent->y, &row, &col);
	gtk_clist_select_row(GTK_CLIST(pdmp3.dir_browser), row, 0);
	l=GTK_MENU_SHELL(pdmp3.dir_browser_menu)->children;
	l=l->next;
	gtk_widget_set_sensitive(GTK_WIDGET(l->data), !play_list_thread_running);
	gtk_menu_popup(GTK_MENU(pdmp3.dir_browser_menu), NULL, NULL, NULL,
		       NULL, bevent->button, bevent->time);
     }
}

/*
 * Callback when a row is selected in
 * our dir_browser
 */

void pdmp3_dir_browser_row_selected_cb(GtkWidget *dir_browser, gint row, gint column, 
				    GdkEventButton *event, gpointer data)
{
   GtkCTreeNode *node;
   GtkDirBrowserNode *dir_node;
   
   if(!pdmp3.in_drag)
     {
	node = gtk_ctree_node_nth(GTK_CTREE(pdmp3.dir_browser), row);
	dir_node = gtk_ctree_node_get_row_data(GTK_CTREE(pdmp3.dir_browser), node);
	pdmp3.selected_dir = dir_node->path;
	pdmp3.selected_dir_browser_row = row;
	pdmp3.selected_file_list_row = 0;
	pdmp3.selected_file = NULL;
	pdmp3_update_menubar();
	pdmp3_update_toolbar();
	pdmp3_clear_entries();
	pdmp3_load_file_list();
	gnome_appbar_set_status(GNOME_APPBAR(pdmp3.statusbar), "");
     }
}

void pdmp3_clear_cb(GtkWidget *widget, GdkEvent *event, gpointer data)
{
   pdmp3_clear_entries();
}

void pdmp3_apply_cb(GtkWidget *widget, GdkEvent *event, gpointer data)
{
   gint retval = 0;

   if(pdmp3.selected_file)
     {
	if(pdmp3.prefs.apply_confirm == TRUE) 
	  {
	     retval = pdmp3_question_dialog(_("Are you sure you want to apply the current tag?"));
	     if(retval)
	       return;
	     else
	       pdmp3_apply_tag();
	  }
	else
	  pdmp3_apply_tag();
	pdmp3_update_row(pdmp3.selected_file_list_row);
     }
}

/*
 * Callback for when the user choses to 
 * revert the tag 
 */

void pdmp3_revert_cb(GtkWidget *widget, GdkEvent *event, gpointer data)
{
   gint retval = 0;
   
   if(pdmp3.selected_file)
     {
	if(pdmp3.prefs.revert_confirm == TRUE)
	  {
	     retval = pdmp3_question_dialog(_("Are you sure you want to revert back to the original tag?"));
	     if(retval)
	       return;
	     else
	       {
		  pdmp3_revert_tag();
	       }
	  }
	else
	  {
	     pdmp3_revert_tag();
	  }
	pdmp3_update_row(pdmp3.selected_file_list_row);
     }
}

/*
 * Generic file deletion callback
 */

void pdmp3_delete_cb(GtkWidget *widget, GdkEvent *event, gpointer data)
{
   gint retval = 0;
   
   if(pdmp3.selected_file)
     {
	if(pdmp3.prefs.delete_confirm == TRUE)
	  {
	     retval = pdmp3_question_dialog(_("Are you sure you want to delete the selected file?"));
	     if(retval)
	       return;
	     else
	       {
		  pdmp3_delete_file();
		  pdmp3.selected_file = NULL;
		  pdmp3_clear_entries();
	       }
	  }
	else
	  {
	     pdmp3_delete_file();
	     pdmp3.selected_file = NULL;
	     pdmp3_clear_entries();
	  }
	pdmp3_update_menubar();
	pdmp3_update_toolbar();
	pdmp3_load_file_list();
	gtk_clist_select_row(GTK_CLIST(pdmp3.file_list), 0, 0);
	gnome_appbar_set_status(GNOME_APPBAR(pdmp3.statusbar), PACKAGE);
     }
}

/*
 * Callback that just brings up the 
 * prefs window
 */

void pdmp3_preferences_cb(GtkWidget *widget, GdkEvent *event, gpointer data)
{
   pdmp3_preferences_window_create();
}

/*
 * Callback to rescan the selected
 * directory for changes
 */

void pdmp3_rescan_cb(GtkWidget *widget, GdkEvent *event, gpointer data)
{
   pdmp3.selected_file_list_row = 0;
   pdmp3.selected_file = NULL;
   pdmp3_update_menubar();
   pdmp3_update_menubar();
   pdmp3_clear_entries();
   pdmp3_load_file_list();
   gnome_appbar_set_status(GNOME_APPBAR(pdmp3.statusbar), PACKAGE);
}

/*
 * Callback for when the starting directory
 * check button is clicked in the prefs
 */

void pdmp3_start_dir_check_button_cb (GtkWidget *widget, GdkEvent *event, gpointer data)
{
   gnome_property_box_changed(GNOME_PROPERTY_BOX(pdmp3.prop_dlg.main_window));
   if (GTK_TOGGLE_BUTTON (widget)->active) 
     {
	gtk_widget_set_sensitive(pdmp3.prop_dlg.start_dir_label, TRUE);
	gtk_widget_set_sensitive(pdmp3.prop_dlg.start_dir_entry, TRUE);
     } else {
        gtk_widget_set_sensitive(pdmp3.prop_dlg.start_dir_label, FALSE);
	gtk_widget_set_sensitive(pdmp3.prop_dlg.start_dir_entry, FALSE);
     }
}

/*
 * Callback for when the apply confirm
 * check button is clicked in the prefs
 */

void pdmp3_apply_confirm_check_button_cb(GtkWidget *widget, GdkEvent *event, gpointer data)
{
   gnome_property_box_changed(GNOME_PROPERTY_BOX(pdmp3.prop_dlg.main_window));
}

/*
 * Callback for when the revert confirm
 * check button is clicked in the prefs
 */

void pdmp3_revert_check_button_cb(GtkWidget *widget, GdkEvent *event, gpointer data)
{
   gnome_property_box_changed(GNOME_PROPERTY_BOX(pdmp3.prop_dlg.main_window));
}

/*
 * Callback for when the move confirm
 * check button is clicked in the prefs
 */

void pdmp3_move_confirm_check_button_cb(GtkWidget *widget, GdkEvent *event, gpointer data)
{
   gnome_property_box_changed(GNOME_PROPERTY_BOX(pdmp3.prop_dlg.main_window));
}

/*
 * Callback for when the delete confirm
 * check button is clicked in the prefs
 */

void pdmp3_delete_confirm_check_button_cb(GtkWidget *widget, GdkEvent *event, gpointer data)
{
   gnome_property_box_changed(GNOME_PROPERTY_BOX(pdmp3.prop_dlg.main_window));
}

void pdpm3_auto_rename_recurse_check_button_cb(GtkWidget *widget, GdkEvent *event, gpointer data)
{
   gnome_property_box_changed(GNOME_PROPERTY_BOX(pdmp3.prop_dlg.main_window));
}

void pdmp3_auto_rename_confirm_check_button_cb(GtkWidget *widget, GdkEvent *event, gpointer data)
{
   gnome_property_box_changed(GNOME_PROPERTY_BOX(pdmp3.prop_dlg.main_window));
}

/*
 * Callback to apply our preferences
 */

void pdmp3_preferences_apply_cb(GnomePropertyBox *widget, gint page, gpointer data)
{
   gchar *temp;
   
   if(page != -1)
     return;
   
   if(GTK_TOGGLE_BUTTON(pdmp3.prop_dlg.override_start_dir_check_button)->active)
     {
	pdmp3.prefs.override_lastdir = 1;
	temp = gtk_entry_get_text(GTK_ENTRY(pdmp3.prop_dlg.start_dir_entry));
	pdmp3.prefs.starting_dir = g_strdup_printf("%s", temp);
     }
   else
     {
	pdmp3.prefs.override_lastdir = 0;
	pdmp3.prefs.starting_dir = g_strdup_printf("%s", pdmp3.selected_dir);
     }
   
   if(GTK_TOGGLE_BUTTON(pdmp3.prop_dlg.apply_confirm_check_button)->active)
     pdmp3.prefs.apply_confirm = TRUE;
   else
     pdmp3.prefs.apply_confirm = FALSE;
   
   if(GTK_TOGGLE_BUTTON(pdmp3.prop_dlg.revert_confirm_check_button)->active)
     pdmp3.prefs.revert_confirm = TRUE;
   else
     pdmp3.prefs.revert_confirm = FALSE;
   
   if(GTK_TOGGLE_BUTTON(pdmp3.prop_dlg.delete_confirm_check_button)->active)
     pdmp3.prefs.delete_confirm = TRUE;
   else
     pdmp3.prefs.delete_confirm = FALSE;

   if(GTK_TOGGLE_BUTTON(pdmp3.prop_dlg.move_confirm_check_button)->active)
     pdmp3.prefs.move_confirm = TRUE;
   else
     pdmp3.prefs.move_confirm = FALSE;

   if(GTK_TOGGLE_BUTTON(pdmp3.prop_dlg.auto_rename_recurse_check_button)->active)
     pdmp3.prefs.auto_rename_recurse = TRUE;
   else
     pdmp3.prefs.auto_rename_recurse = FALSE;
   
   if(GTK_TOGGLE_BUTTON(pdmp3.prop_dlg.auto_rename_confirm_check_button)->active)
     pdmp3.prefs.auto_rename_confirm = TRUE;
   else
     pdmp3.prefs.auto_rename_confirm = FALSE;
}

/*
 * Callback to access our homepage
 */

void pdmp3_homepage_url_cb(GtkWidget *widget, const char *url)
{
   gnome_url_show(url);
}

/*
 * Callback to bring up our file
 * rename dialog
 */

void pdmp3_rename_cb(GtkWidget *widget, GdkEvent *event, gpointer data)
{
   if(pdmp3.selected_file)
     {
	pdmp3_rename_dialog_new();
     }
   pdmp3_update_menubar();
   pdmp3_update_toolbar();
}

/*
 * Callback to bring up our file
 * info dialog
 */

void pdmp3_file_info_cb(GtkWidget *widget, GdkEvent *event, gpointer data)
{
   if(pdmp3.selected_file)
     {
	pdmp3_info_window_new();
     }
}

/*
 * Callback to launch an external program
 */

void pdmp3_launch_cb(GtkWidget *widget, gpointer data)
{
   gchar *program;

   program = g_strdup_printf("%s", (gchar *)data);
   pdmp3_launch_program(program);
}

/* Callback to open our play list
 * creation dialog */

void pdmp3_play_list_cb(GtkWidget *widget, GdkEvent *event, gpointer data)
{
   pdmp3_play_list_dlg_new();
}

/*
 * Callback to keep track of where we are
 * dragging
 */

void pdmp3_drag_motion_cb(GtkWidget *widget, GdkDragContext *drag_context,
			  gint x, gint y, guint time, gpointer data)
{
   int row = 0;
   int col = 0;
   
   gtk_clist_get_selection_info(GTK_CLIST(pdmp3.dir_browser),
				x, y, &row, &col);
   gtk_clist_select_row(GTK_CLIST(pdmp3.dir_browser), row, 0);
}

/*
 * Callback that just sets our in_drag flag
 * to true
 */

void pdmp3_drag_begin_cb(GtkWidget *widget, GdkDragContext *drag_context,
		      gpointer data)
{
   pdmp3.in_drag = TRUE;
}

/*
 * Callback to clean up after a drop operation
 * has been completed
 */

void pdmp3_drag_end_cb(GtkWidget *widget, GdkDragContext *drag_context,
		       gpointer data)
{
   pdmp3.in_drag = FALSE;
   
   if(pdmp3.selected_dir_browser_row == -1)
     gtk_dir_browser_open_dir(GTK_DIR_BROWSER(pdmp3.dir_browser), pdmp3.selected_dir);
   else
     gtk_clist_select_row(GTK_CLIST(pdmp3.dir_browser), pdmp3.selected_dir_browser_row, 0);
}

/*
 * Callback for when we drop our file into
 * one of our folders
 */

void pdmp3_dir_browser_drop_cb(GtkWidget *widget, GdkDragContext  *context, gint x, gint y,
			       GtkSelectionData *data, guint info, guint time)
{
   gint row = 0;
   gint col = 0;
   GtkCTreeNode *node;
   GtkDirBrowserNode *dir_node;
   gchar *dest_dir;
   gchar *dest_file;
   gchar *source_file;
   
   if(pdmp3.selected_file)
     {
	gtk_clist_get_selection_info(GTK_CLIST(pdmp3.dir_browser),
				     x, y, &row, &col);
	node = gtk_ctree_node_nth(GTK_CTREE(pdmp3.dir_browser), row);
	dir_node = gtk_ctree_node_get_row_data(GTK_CTREE(pdmp3.dir_browser), node);
	dest_dir = dir_node->path;
	dest_file = g_strdup_printf("%s/%s", dest_dir, g_basename(pdmp3.selected_file));
	source_file = g_strdup_printf("%s", pdmp3.selected_file);
	
	if(pdmp3.prefs.move_confirm == TRUE)
	  {
	     gint retval = 0;
	     retval = pdmp3_question_dialog(_("Are you sure you want to move the selected file?"));
	     if(retval)
	       return;
	     else
	       {
		  pdmp3_rename_file(source_file, dest_file);
		  pdmp3.selected_file = NULL;
		  pdmp3_load_file_list();
		  gtk_clist_select_row(GTK_CLIST(pdmp3.file_list), 0, 0);
	       }
	  }
	else
	  {
	     pdmp3_rename_file(source_file, dest_file);
	     pdmp3.selected_file = NULL;
	     pdmp3_load_file_list();
	     gtk_clist_select_row(GTK_CLIST(pdmp3.file_list), 0, 0);
	  }
     }
   if(!pdmp3.selected_file)
     {
	gnome_appbar_set_status(GNOME_APPBAR(pdmp3.statusbar), PACKAGE);
	pdmp3_clear_entries();
     }
   pdmp3_update_menubar();
   pdmp3_update_toolbar();
}

void pdmp3_space_to_underscore_cb(GtkWidget *widget, gpointer data)
{
   if(pdmp3.prefs.auto_rename_confirm == TRUE)
     {
	gint retval = 0;
	retval = pdmp3_question_dialog(_("Are you sure you want to auto rename this directory?"));
	if(retval)
	  return;
	else
	  pdmp3_space_to_underscore();
     }
   else
     pdmp3_space_to_underscore();
}

void pdmp3_underscore_to_space_cb(GtkWidget *widget, gpointer data)
{
   if(pdmp3.prefs.auto_rename_confirm == TRUE)
     {
	gint retval = 0;
	retval = pdmp3_question_dialog(_("Are you sure you want to auto rename this directory?"));
	if(retval)
	  return;
	else
	  pdmp3_underscore_to_space();
     }
   else
     pdmp3_underscore_to_space();
}
