/*****************************************************************************
 * PDMP3 - A MP3 manager for GNOME                                           *
 * Copyright (C) 2000 Scott Guilbeaux <scottg@viptx.net>                     *
 *                                                                           *
 * This program is free software; you can redistribute it and/or modify      *
 * it under the terms of the GNU General Public License as published by      *
 * the Free Software Foundation; either version 2 of the License, or         *
 * (at your option) any later version.                                       *
 *                                                                           *
 * This program is distributed in the hope that it will be useful,           *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of            *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
 * GNU General Public License for more details.                              *
 *                                                                           *
 * You should have received a copy of the GNU General Public License         *
 * along with this program; if not, write to the Free Software               *
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA *
 *****************************************************************************/

#include "pdmp3.h"
#include "pixmaps/check.xpm"
#include "pixmaps/x.xpm"

/*
 * Scan the direcotory for mp3s and load them into
 * the clist
 */

void pdmp3_load_file_list(void)
{
   DIR *dir;
   gchar *group[4];
   gchar *error;
   gchar *full_path;
   const gchar *ext;
   gint row = 0;
   struct dirent *dirent;
   struct stat sb;
   GnomePixmap *check_pixmap;
   GnomePixmap *x_pixmap;
      
   /* get our pixmaps ready */
   check_pixmap = (GnomePixmap *)gnome_pixmap_new_from_xpm_d(check_xpm);
   x_pixmap = (GnomePixmap *)gnome_pixmap_new_from_xpm_d(x_xpm);
   
   gtk_clist_freeze(GTK_CLIST(pdmp3.file_list));
   
   dir = opendir(pdmp3.selected_dir);
   if(dir)
     {
	gtk_clist_clear(GTK_CLIST(pdmp3.file_list));

	while((dirent = readdir(dir)))
	  {
	     if((g_strncasecmp(dirent->d_name, ".", 1) != 0) && (g_strncasecmp(dirent->d_name, "..", 2) != 0))
	       {
		  full_path = g_strdup_printf("%s%s", pdmp3.selected_dir, dirent->d_name);
		  stat(full_path, &sb);
		  if(!S_ISDIR(sb.st_mode))
		    {
		       ext = g_extension_pointer(dirent->d_name);
		       if((g_strncasecmp(ext, "mp3", 3) == 0))
			 {
			    group[0] = NULL;
			    group[1] = dirent->d_name;
			    group[2] = g_strdup_printf("%d", (int)sb.st_size);
			    group[3] = ctime(&sb.st_mtime);
			    gtk_clist_append(GTK_CLIST(pdmp3.file_list), group);
			    if(id3_check_for_tag(full_path))
			      {
				 gtk_clist_set_pixmap(GTK_CLIST(pdmp3.file_list), row, 0, 
						      check_pixmap->pixmap, 
						      check_pixmap->mask);
			      }
			    else
			      {
				 gtk_clist_set_pixmap(GTK_CLIST(pdmp3.file_list), row, 0, 
						      x_pixmap->pixmap, 
						      x_pixmap->mask);
			      }
			    gtk_clist_set_row_data(GTK_CLIST(pdmp3.file_list), row, (gpointer)pdmp3.selected_dir);
			    row++;
			 }
		    }
	       }
	  }
	closedir(dir);
     }
   else
     {
	error = g_strdup_printf(_("Could not open %s for reading: %s"), pdmp3.selected_dir, g_strerror(errno));
	gnome_error_dialog_parented(error, GTK_WINDOW(pdmp3.main_window));
	gtk_clist_clear(GTK_CLIST(pdmp3.file_list));
     }
   gtk_clist_thaw(GTK_CLIST(pdmp3.file_list));
   if(pdmp3.selected_file)
     gtk_clist_select_row(GTK_CLIST(pdmp3.file_list), pdmp3.selected_file_list_row, 0);
}

/* 
 * Updates the info in the file list 
 */
void pdmp3_update_row(gint row)
{
   gpointer ptr = NULL;
   gchar *path;
   gchar *longname;
   gchar *filename;
   gchar *temp;
   struct stat sb;
   GnomePixmap *check_pixmap;
   GnomePixmap *x_pixmap;

   check_pixmap = (GnomePixmap *)gnome_pixmap_new_from_xpm_d(check_xpm);
   x_pixmap = (GnomePixmap *)gnome_pixmap_new_from_xpm_d(x_xpm);
   
   pdmp3.selected_file_list_row = row;
   
   ptr = gtk_clist_get_row_data(GTK_CLIST(pdmp3.file_list), row);
   path = g_strdup_printf("%s", (char *)ptr);
   gtk_clist_get_text(GTK_CLIST(pdmp3.file_list), row, 1, &filename);
   longname = g_strdup_printf("%s%s", path, filename);
   stat(longname, &sb);
   temp = g_strdup_printf("%d", (int)sb.st_size);

   if(id3_check_for_tag(longname))
     {
	gtk_clist_set_pixmap(GTK_CLIST(pdmp3.file_list), row, 0, 
			     check_pixmap->pixmap, 
			     check_pixmap->mask);
     }
   else
     {
	gtk_clist_set_pixmap(GTK_CLIST(pdmp3.file_list), row, 0, 
			     x_pixmap->pixmap, 
			     x_pixmap->mask);
     }
   gtk_clist_set_text(GTK_CLIST(pdmp3.file_list), row, 1, g_basename(longname));
   gtk_clist_set_text(GTK_CLIST(pdmp3.file_list), row, 2, temp);
   gtk_clist_set_text(GTK_CLIST(pdmp3.file_list), row, 3, ctime(&sb.st_mtime));
}
