/*****************************************************************************
 * PDID3 - An ID3 tag editor for GNOME                                        *
 * Copyright (C) 2000 Scott Guilbeaux <scottg@viptx.net>                     *
 *                                                                           *
 * This program is free software; you can redistribute it and/or modify      *
 * it under the terms of the GNU General Public License as published by      *
 * the Free Software Foundation; either version 2 of the License, or         *
 * (at your option) any later version.                                       *
 *                                                                           *
 * This program is distributed in the hope that it will be useful,           *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of            *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
 * GNU General Public License for more details.                              *
 *                                                                           *
 * You should have received a copy of the GNU General Public License         *
 * along with this program; if not, write to the Free Software               *
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA *
 *****************************************************************************/

#include <gtk/gtk.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <dirent.h>
#include <string.h>

#include "gtkdirbrowser.h"
#include "pixmaps/mycomputer.xpm"
#include "pixmaps/folder_open.xpm"
#include "pixmaps/folder_closed.xpm"
#include "pixmaps/folder_deny.xpm"

#define MAXHOSTNAMELEN 64

static GtkCTreeClass *parent_class = NULL;

static void destroy_tree(gpointer data);
static void expand_tree(GtkCTree *ctree, GtkCTreeNode *parent_node);

static void gtk_dir_browser_class_init(GtkDirBrowserClass *);
static void gtk_dir_browser_init(GtkDirBrowser *);

guint gtk_dir_browser_get_type(void)
{
   static GtkType dir_browser_type = 0;
   
   if(!dir_browser_type)
     {
	GtkTypeInfo dir_browser_info = 
	  {
	     "GtkDirBrowser",
	      sizeof(GtkDirBrowser),
	      sizeof(GtkDirBrowserClass),
	      (GtkClassInitFunc) gtk_dir_browser_class_init,
	      (GtkObjectInitFunc) gtk_dir_browser_init,
	      (GtkArgSetFunc) NULL,
	      (GtkArgGetFunc) NULL
	  };
	
	dir_browser_type = gtk_type_unique(gtk_ctree_get_type(), &dir_browser_info);
     }
   return dir_browser_type;
}

static void gtk_dir_browser_class_init(GtkDirBrowserClass *class)
{
   GtkObjectClass *object_class;
   
   object_class = (GtkObjectClass*) class;

   parent_class = gtk_type_class(gtk_ctree_get_type());
}

static void gtk_dir_browser_init(GtkDirBrowser *dir_browser)
{
   GtkWidget *widget; 
   GtkCTreeNode *root_node,*mycomputer_node,*node;
   GtkDirBrowserNode *dirnode;
   gchar *root_text="/", *node_text="dummy";
   GdkColormap *colormap;
   gchar localhost[MAXHOSTNAMELEN];
   
   widget = GTK_WIDGET(dir_browser);
   colormap = gdk_colormap_get_system();

   if((gethostname(localhost, MAXHOSTNAMELEN) != 0))
     strcpy(localhost, "My Computer");
   dir_browser->hostname = g_strdup(localhost);
   
   gtk_ctree_construct(GTK_CTREE(dir_browser), 1, 0, NULL);
   
   gtk_clist_set_row_height(GTK_CLIST(dir_browser), 20);
   
   dir_browser->mycomputer = gdk_pixmap_colormap_create_from_xpm_d(NULL, colormap,
								   &dir_browser->mycomputer_mask,
								   NULL, mycomputer_xpm);
   dir_browser->folder_open = gdk_pixmap_colormap_create_from_xpm_d(NULL, colormap,
								    &dir_browser->folder_open_mask,
								    NULL, folder_open_xpm);
   dir_browser->folder_closed = gdk_pixmap_colormap_create_from_xpm_d(NULL, colormap,
								    &dir_browser->folder_closed_mask,
								    NULL, folder_closed_xpm);
   dir_browser->folder_deny = gdk_pixmap_colormap_create_from_xpm_d(NULL, colormap,
								    &dir_browser->folder_deny_mask,
								    NULL, folder_deny_xpm);
   
   gtk_clist_set_column_auto_resize(GTK_CLIST(dir_browser), 0, TRUE);
   gtk_clist_set_selection_mode(GTK_CLIST(dir_browser), GTK_SELECTION_SINGLE);
   gtk_ctree_set_line_style(GTK_CTREE(dir_browser), GTK_CTREE_LINES_DOTTED);
   gtk_signal_connect(GTK_OBJECT(dir_browser), "tree_expand", 
		      GTK_SIGNAL_FUNC(expand_tree), NULL);
   
   mycomputer_node = gtk_ctree_insert_node(GTK_CTREE(dir_browser), NULL, NULL, &dir_browser->hostname, 4, dir_browser->mycomputer, dir_browser->mycomputer_mask, dir_browser->mycomputer, dir_browser->mycomputer_mask, FALSE, FALSE);
   
   dirnode = g_malloc0(sizeof(GtkDirBrowserNode));
   dirnode->path = dir_browser->hostname;
   gtk_ctree_node_set_row_data_full(GTK_CTREE(dir_browser), mycomputer_node,dirnode, destroy_tree);
   
   root_node = gtk_ctree_insert_node(GTK_CTREE(dir_browser), mycomputer_node, NULL, &root_text, 4, dir_browser->folder_closed, dir_browser->folder_closed_mask, dir_browser->folder_open, dir_browser->folder_open_mask, FALSE, FALSE);
   
   dirnode = g_malloc0(sizeof(GtkDirBrowserNode));
   dirnode->path = g_strdup("/");
   gtk_ctree_node_set_row_data_full(GTK_CTREE(dir_browser), root_node, dirnode, destroy_tree);
   node = gtk_ctree_insert_node(GTK_CTREE(dir_browser), root_node, NULL, &node_text, 4, NULL, NULL, NULL, NULL, TRUE, TRUE);
   gtk_ctree_expand(GTK_CTREE(dir_browser), mycomputer_node);
   gtk_ctree_select(GTK_CTREE(dir_browser), root_node);
}

GtkWidget *gtk_dir_browser_new(void)
{
   return GTK_WIDGET(gtk_type_new(gtk_dir_browser_get_type()));
}

static gboolean check_for_subdir(gchar *path)
{
   DIR *dir;
   struct dirent *dent;
   struct stat sb;
   gchar *npath;
   
   if((dir = opendir(path))!=NULL)
     {
	while((dent=readdir(dir))!=NULL)
	  {
	     if(dent->d_name[0]!='.')
	       {
		  npath=g_strconcat(path,dent->d_name, "/", NULL);
		  stat(npath, &sb);
		  g_free(npath);
		  if(S_ISDIR(sb.st_mode))
		    {
		       closedir(dir);
		       return TRUE;
		    }
	       }
	  }
	closedir(dir);
     }
   return FALSE;
}

static void destroy_tree(gpointer data)
{
   GtkDirBrowserNode *node;
   node = data;
   g_free(node->path);
   g_free(node);
}

static void expand_tree(GtkCTree *ctree, GtkCTreeNode *parent_node)
{
   DIR *dir;
   struct dirent *dent;
   gchar *path, *text, *dummy="dummy";
   struct stat sb;
   GtkCTreeNode *node, *sub_node;
   GtkDirBrowserNode  *parent_dirnode, *dirnode;
   gboolean has_subdir = FALSE;
   GtkDirBrowser *dir_browser;
   GtkWidget *widget;
   
   widget = GTK_WIDGET(ctree);
   dir_browser = GTK_DIR_BROWSER(widget);

   parent_dirnode = gtk_ctree_node_get_row_data(GTK_CTREE(widget), parent_node);
   
   if(parent_dirnode->path == dir_browser->hostname)
     return;
   
   if(!parent_dirnode->scanned)
     {
	gtk_clist_freeze(GTK_CLIST(widget));
	node = gtk_ctree_find_by_row_data(GTK_CTREE(widget), parent_node, NULL);
	gtk_ctree_remove_node(GTK_CTREE(widget), node);
	if((dir=opendir(parent_dirnode->path))!=NULL)
	  {
	     while((dent=readdir(dir))!=NULL)
	       {
		  path = g_strconcat(parent_dirnode->path, dent->d_name, NULL);
		  stat(path, &sb);
		  if(S_ISDIR(sb.st_mode) && dent->d_name[0] != '.')
		    {
		       DIR *dir_sub;
		       dirnode = g_malloc0(sizeof(GtkDirBrowserNode));
		       dirnode->path = g_strconcat(path, "/", NULL);
		       text = dent->d_name;
		       if(check_for_subdir(dirnode->path))
			 has_subdir = TRUE;
		       else
			 has_subdir = FALSE;
		       dir_sub = opendir(dirnode->path);
		       if(dir_sub != NULL)
			 {
			    closedir(dir_sub);
			    node = gtk_ctree_insert_node(GTK_CTREE(widget), parent_node, NULL, &text, 4, dir_browser->folder_closed, dir_browser->folder_closed_mask, dir_browser->folder_open, dir_browser->folder_open_mask, !has_subdir, FALSE);
			 }
		       else
			 {
			    node = gtk_ctree_insert_node(GTK_CTREE(widget), parent_node, NULL, &text, 4, dir_browser->folder_deny, dir_browser->folder_deny_mask, dir_browser->folder_deny, dir_browser->folder_deny_mask, !has_subdir, FALSE);
			 }
		       gtk_ctree_node_set_row_data_full(GTK_CTREE(widget), node, dirnode, destroy_tree);
		       if(has_subdir)
			 sub_node = gtk_ctree_insert_node(GTK_CTREE(widget), node, NULL, &dummy, 4, NULL, NULL, NULL, NULL, FALSE, FALSE);
		    }
		  g_free(path);
	       }
	     closedir(dir);
	     gtk_ctree_sort_node(GTK_CTREE(widget), parent_node);
	  }
	gtk_clist_thaw(GTK_CLIST(widget));
	parent_dirnode->scanned = TRUE;
     }
}

gint gtk_dir_browser_open_dir(GtkDirBrowser *dir_browser, const gchar *path)
{
   GtkCTreeNode *root_node, *node;
   GtkDirBrowserNode *dir_node;
   DIR *dir;
   const gchar *c;
   gchar *folder;
   gint nlen;
   gint new_path, new_node;
   gchar *text;
   
   if((dir=opendir(path)) == NULL) return FALSE;
   closedir(dir);
   
   c = path;
   folder = NULL;
   nlen = 0;
   
   root_node = gtk_ctree_node_nth(GTK_CTREE(dir_browser), 1);
   gtk_ctree_expand(GTK_CTREE(dir_browser), root_node);
   
   new_path = FALSE;
   new_node = TRUE;
   
   while(*c != '\0' && *c != '\n' && c != NULL){
      nlen++;
      folder = (char *)g_realloc(folder, (nlen+1)*sizeof(char));
      folder[nlen-1] = *c;
      folder[nlen]='\0';
      if(*c == '/'){
	 if(new_path){
	    node = GTK_CTREE_ROW(root_node)->children;
	    while(node){
	       dir_node = gtk_ctree_node_get_row_data(GTK_CTREE(dir_browser), node);
	       text = dir_node->path;
	       if(strcmp(text, folder) == 0){
		  gtk_ctree_expand(GTK_CTREE(dir_browser), node);
		  root_node = node;
		  break;
	       }
	       node = GTK_CTREE_NODE_NEXT(node);
	    }
	 }
	 else
	   {
	      new_path = TRUE;
	   }
	 new_node = FALSE;
      } else {
	 new_node = TRUE;
      }
      c++;
   }
   
   if(new_node){
	nlen++;
	folder = (char *)g_realloc(folder, (nlen+1)*sizeof(char));
	folder[nlen-1] = '/';
	folder[nlen]='\0';
	node = GTK_CTREE_ROW(root_node)->children;
	while(node){
	       dir_node = gtk_ctree_node_get_row_data(GTK_CTREE(dir_browser), node);
	       text = dir_node->path;
	       if(strcmp(text, folder) == 0){
			 gtk_ctree_expand(GTK_CTREE(dir_browser), node);
			 root_node = node;
			 break;
		      }
	       node = GTK_CTREE_NODE_NEXT(node);
	    }
     }
   
   g_free(folder);
   gtk_ctree_select(GTK_CTREE(dir_browser), root_node);
   return TRUE;
}
