/*****************************************************************************
 * PDMP3 - A MP3 manager for GNOME                                           *
 * Copyright (C) 2000 Scott Guilbeaux <scottg@viptx.net>                     *
 *                                                                           *
 * This program is free software; you can redistribute it and/or modify      *
 * it under the terms of the GNU General Public License as published by      *
 * the Free Software Foundation; either version 2 of the License, or         *
 * (at your option) any later version.                                       *
 *                                                                           *
 * This program is distributed in the hope that it will be useful,           *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of            *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
 * GNU General Public License for more details.                              *
 *                                                                           *
 * You should have received a copy of the GNU General Public License         *
 * along with this program; if not, write to the Free Software               *
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA *
 *****************************************************************************/

#include "pdmp3.h"

/*
 * Create our data storage directory
 * if it doesn't exist
 */

void pdmp3_setup_data_dir(void)
{
   gchar *str;
   DIR *dir;
   
   str = gnome_util_prepend_user_home(".pdmp3/");
   dir = opendir(str);
   if(!dir)
     {
	mkdir(str, 0755);
     }
}

/*
 * Initialise some variables and configuration
 * options
 */

void pdmp3_init(void)
{
   pdmp3_setup_data_dir();
   pdmp3_read_config_options();
   pdmp3.selected_dir = g_strdup_printf("%s", pdmp3.prefs.starting_dir);
   pdmp3.selected_file = NULL;
   pdmp3.selected_file_list_row = -1;
   pdmp3.selected_dir_browser_row = -1;
   pdmp3.in_drag = FALSE;
   play_list_thread_running = FALSE;
   rename_thread_running = FALSE;
}

/*
 * Get configuration options from our config file
 */

void pdmp3_read_config_options(void)
{
   gchar *text;

   pdmp3.prefs.apply_confirm = gnome_config_get_int_with_default("/pdmp3/confirmations/apply_confirm=0", NULL);
   pdmp3.prefs.override_lastdir = gnome_config_get_int_with_default("/pdmp3/environment/override_lastdir=0", NULL);
   pdmp3.prefs.revert_confirm = gnome_config_get_int_with_default("/pdmp3/confirmations/revert_confirm=1", NULL);
   pdmp3.prefs.delete_confirm = gnome_config_get_int_with_default("/pdmp3/confirmations/delete_confirm=1", NULL);
   pdmp3.prefs.move_confirm = gnome_config_get_int_with_default("/pdmp3/confirmations/move_confirm=1", NULL);
   pdmp3.prefs.auto_rename_recurse = gnome_config_get_int_with_default("/pdmp3/auto-rename/recurse=1", NULL);
   pdmp3.prefs.auto_rename_confirm = gnome_config_get_int_with_default("/pdmp3/auto-rename/confirm=1", NULL);
   text = gnome_config_get_string("/pdmp3/environment/starting_directory");
   if(!text)
     pdmp3.prefs.starting_dir = g_strdup_printf("%s", gnome_util_user_home());
   else
     pdmp3.prefs.starting_dir = g_strdup_printf("%s", text);
   
   g_free(text);
}
