/*****************************************************************************
 * PDMP3 - An MP3 manager for GNOME                                          *
 * Copyright (C) 2000 Scott Guilbeaux <scottg@viptx.net>                     *
 *                                                                           *
 * This program is free software; you can redistribute it and/or modify      *
 * it under the terms of the GNU General Public License as published by      *
 * the Free Software Foundation; either version 2 of the License, or         *
 * (at your option) any later version.                                       *
 *                                                                           *
 * This program is distributed in the hope that it will be useful,           *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of            *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
 * GNU General Public License for more details.                              *
 *                                                                           *
 * You should have received a copy of the GNU General Public License         *
 * along with this program; if not, write to the Free Software               *
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA *
 *****************************************************************************/

#include "pdmp3.h"
#include "pixmaps/apply.xpm"
#include "pixmaps/clear.xpm"
#include "pixmaps/scan.xpm"
#include "pixmaps/xmms.xpm"
#include "pixmaps/gnapster.xpm"
#include "pixmaps/ripperx.xpm"

/* 
 * Menubar stuff 
 */

/* Auto rename submenu */
GnomeUIInfo auto_rename_menu[] = {
     {
	GNOME_APP_UI_ITEM,
	N_("Convert Spaces to Underscores"),
	N_("Convert spaces in file names to underscores"),
	pdmp3_space_to_underscore_cb,
     },
     {
	GNOME_APP_UI_ITEM,
	N_("Convert Underscores to Spaces"),
	N_("Convert underscores in file names to spaces"),
	pdmp3_underscore_to_space_cb,
     },
   GNOMEUIINFO_END
};

/* File menu */

GnomeUIInfo file_menu[] = {
     {
	GNOME_APP_UI_ITEM,
	N_("_Info"),
	N_("Information about the selected file"),
	pdmp3_file_info_cb,
     },
     {
	GNOME_APP_UI_ITEM,
	N_("_Rename"),
	N_("Rename the selected file"),
	pdmp3_rename_cb,
     },
     {
	GNOME_APP_UI_ITEM,
	N_("_Delete"),
	N_("Delete the selected file"),
	pdmp3_delete_cb, NULL,
	NULL,
	GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_PIXMAP_TRASH,
	'\0', 0, NULL
     },
   GNOMEUIINFO_SEPARATOR,
     {
	GNOME_APP_UI_SUBTREE,
	N_("Auto Rename"),
	N_("Auto renaming"),
	&auto_rename_menu,
     },
     {
	GNOME_APP_UI_ITEM,
	N_("Create XMMS Playlist"),
        N_("Create XMMS playlist from the selected directory"),
        NULL,
     },
   GNOMEUIINFO_SEPARATOR,
   GNOMEUIINFO_MENU_EXIT_ITEM(pdmp3_quit_cb, NULL),
   GNOMEUIINFO_END
};

/* Edit menu */

GnomeUIInfo edit_menu[] = {
     {
	GNOME_APP_UI_ITEM,
	N_("_Apply"),
	N_("Apply the current ID3 tag"),
	pdmp3_apply_cb, NULL,
	NULL,
	GNOME_APP_PIXMAP_DATA, apply_xpm,
	'\0', 0, NULL
     },
     {
	GNOME_APP_UI_ITEM,
	N_("_Clear"),
	N_("Clear the text entries"),
	pdmp3_clear_cb, NULL,
	NULL,
	GNOME_APP_PIXMAP_DATA, clear_xpm,
	'\0', 0, NULL
     },
     {
	GNOME_APP_UI_ITEM,
	N_("Re_vert"),
	N_("Revert back to the original tag"),
	pdmp3_revert_cb, NULL,
	NULL,
	GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_PIXMAP_REVERT,
	'\0', 0, NULL
     },
   GNOMEUIINFO_MENU_PREFERENCES_ITEM(pdmp3_preferences_cb, NULL),
   GNOMEUIINFO_END
};

/* Launch menu */
GnomeUIInfo launch_menu[] = {
#ifdef HAVE_XMMS
     {
	GNOME_APP_UI_ITEM,
	N_("Xmms"),
	N_("Launch xmms"),
	pdmp3_launch_cb, "xmms",
	NULL,
	GNOME_APP_PIXMAP_DATA, xmms_xpm,
	'\0', 0, NULL
     },
#endif
#ifdef HAVE_GNAP
     {
	GNOME_APP_UI_ITEM,
	N_("Gnapster"),
	N_("Launch gnapster"),
	pdmp3_launch_cb, "gnapster",
	NULL,
	GNOME_APP_PIXMAP_DATA, gnapster_xpm,
	'\0', 0, NULL
     },
#endif
#ifdef HAVE_RIPPERX
     {
	GNOME_APP_UI_ITEM,
	N_("RipperX"),
	N_("Launch ripperX"),
	pdmp3_launch_cb, "ripperX",
	NULL,
	GNOME_APP_PIXMAP_DATA, ripperx_xpm,
	'\0', 0, NULL
     },
#endif
#ifdef HAVE_FREEAMP
     {
	GNOME_APP_UI_ITEM,
	N_("Freeamp"),
	N_("Launch freeamp"),
	pdmp3_launch_cb, "freeamp",
     },
#endif
   GNOMEUIINFO_END
};

/* Help menu */

GnomeUIInfo help_menu[] = {
     {
	GNOME_APP_UI_ITEM,
	N_("PDMP3 _homepage"),
	N_("PDMP3 _homepage"),
	pdmp3_homepage_url_cb, HOMEPAGE, NULL,
	GNOME_APP_PIXMAP_STOCK,
	GNOME_STOCK_MENU_HOME,
	'\0', 0, NULL
     },
   GNOMEUIINFO_MENU_ABOUT_ITEM(pdmp3_about_cb, NULL),
   GNOMEUIINFO_END
};

/* Menubar */

GnomeUIInfo menubar[] = {
   GNOMEUIINFO_MENU_FILE_TREE(file_menu),
   GNOMEUIINFO_MENU_EDIT_TREE(edit_menu),
   GNOMEUIINFO_SUBTREE(N_("_Launch"), &launch_menu),
   GNOMEUIINFO_MENU_HELP_TREE(help_menu),
   GNOMEUIINFO_END
};

/* 
 * Toolbar stuff 
 */

GnomeUIInfo toolbar[] = {
   GNOMEUIINFO_ITEM_STOCK(N_("Exit"), N_("Exit the application"),
			  pdmp3_quit_cb, GNOME_STOCK_PIXMAP_EXIT),
     {
	GNOME_APP_UI_ITEM,
	N_("Delete"),
	N_("Delete the selected file"),
	pdmp3_delete_cb, NULL,
	NULL,
	GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_PIXMAP_TRASH,
	'\0', 0, NULL
     },
   {
      GNOME_APP_UI_ITEM,
      N_("Rescan"),
      N_("Rescan directory"),
      pdmp3_rescan_cb, NULL,
      NULL, GNOME_APP_PIXMAP_DATA, scan_xpm,
      '\0', 0, NULL
   },
   GNOMEUIINFO_SEPARATOR,
     {
	GNOME_APP_UI_ITEM,
	N_("Apply"),
	N_("Apply the currend ID3 tag"),
	pdmp3_apply_cb, NULL,
	NULL,
	GNOME_APP_PIXMAP_DATA, apply_xpm,
	'\0', 0, NULL
     },
     {
	GNOME_APP_UI_ITEM,
	N_("Clear"),
	N_("Clear the text entries"),
	pdmp3_clear_cb, NULL,
	NULL,
	GNOME_APP_PIXMAP_DATA, clear_xpm,
	'\0', 0, NULL
     },
     {
	GNOME_APP_UI_ITEM,
	N_("Revert"),
	N_("Revert to the original tag"),
	pdmp3_revert_cb, NULL,
	NULL,
	GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_PIXMAP_REVERT,
	'\0', 0, NULL
     },
   GNOMEUIINFO_END
};

/* Dir browser popup menu */
GnomeUIInfo dir_browser_menu_popup[] = {
   GNOMEUIINFO_SEPARATOR,
     {
	GNOME_APP_UI_SUBTREE,
	N_("Auto Rename"),
	N_("Auto renaming"),
	&auto_rename_menu,
     },
   GNOMEUIINFO_SEPARATOR,
     {
	GNOME_APP_UI_ITEM,
	N_("Create Playlist"),
	N_("Create an XMMS playlist from this directory"),
	pdmp3_play_list_cb
     },
   GNOMEUIINFO_END
};

/* File list menu file file sub tree */
GnomeUIInfo file_list_file_menu[] = {
     {
	GNOME_APP_UI_ITEM,
	N_("Info"),
	N_("MP3 file info"),
	pdmp3_file_info_cb, NULL,
	NULL,
	GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_BLANK,
	'\0', 0, NULL
     },
     {
	GNOME_APP_UI_ITEM,
	N_("Rename"),
	N_("Rename the selected file"),
	pdmp3_rename_cb, NULL,
	NULL,
	GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_BLANK,
	'\0', 0, NULL
     },
     {
	GNOME_APP_UI_ITEM,
	N_("Delete"),
	N_("Delete the selected file"),
	pdmp3_delete_cb, NULL,
	NULL,
	GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_TRASH,
	'\0', 0, NULL
     },
   GNOMEUIINFO_END
};

/* File list menu edit sub tree */
GnomeUIInfo file_list_edit_menu[] = {
     {
	GNOME_APP_UI_ITEM,
	N_("Apply"),
	N_("Apply the current ID3 tag"),
	pdmp3_apply_cb, NULL,
	NULL,
	GNOME_APP_PIXMAP_DATA, apply_xpm,
	'\0', 0, NULL
     },
     {
	GNOME_APP_UI_ITEM,
	N_("Clear"),
	N_("Clear text entry contents"),
	pdmp3_clear_cb, NULL, 
	NULL,
	GNOME_APP_PIXMAP_DATA, clear_xpm,
	'\0', 0, NULL
     },
     {
	GNOME_APP_UI_ITEM,
	N_("Revert"),
	N_("Revert to the original tag"),
	pdmp3_clear_cb, NULL,
	NULL,
	GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_PIXMAP_REVERT,
	'\0', 0, NULL
     },
   GNOMEUIINFO_END
};

/* File list popup menu */
GnomeUIInfo file_list_menu_popup[] = {
     {
	GNOME_APP_UI_SUBTREE,
	N_("File"),
        N_("File"),
	&file_list_file_menu, NULL,
	NULL,
	GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_BLANK,
	'\0', 0, NULL
     },
     GNOMEUIINFO_SEPARATOR,
     {
	GNOME_APP_UI_SUBTREE,
	N_("Edit"),
	N_("Edit"),
	&file_list_edit_menu, NULL,
	NULL,
	GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_BLANK,
	'\0', 0, NULL
     },
   GNOMEUIINFO_END
};

/* 
 * Convenience function to create a regular gtk button
 * with a pixmap on it (unused)
 */

GtkWidget *gnome_pixmap_button_new_with_label(gchar **data, gchar *text, int width,
					      int height, int padding)
{
   GtkWidget *button;
   GtkWidget *hbox;
   GtkWidget *pixmap;
   GtkWidget *label;

   hbox = gtk_hbox_new(FALSE, 0);
   pixmap = gnome_pixmap_new_from_xpm_d_at_size(data, width, height);
   gtk_box_pack_start(GTK_BOX(hbox), pixmap, TRUE, TRUE, padding);
   label = gtk_label_new(text);
   gtk_box_pack_end(GTK_BOX(hbox), label, TRUE, TRUE, padding);
   button = gtk_button_new();
   gtk_container_border_width(GTK_CONTAINER(button), padding);
   gtk_container_add(GTK_CONTAINER(button), hbox);

   return(button);
}

/*
 * Convenience function to clear out
 * the contents of our text fields
 */

void pdmp3_clear_entries(void)
{
   gtk_entry_set_text(GTK_ENTRY(pdmp3.artist_entry), "");
   gtk_entry_set_text(GTK_ENTRY(pdmp3.title_entry), "");
   gtk_entry_set_text(GTK_ENTRY(pdmp3.album_entry), "");
   gtk_entry_set_text(GTK_ENTRY(pdmp3.year_entry), "");
   gtk_entry_set_text(GTK_ENTRY(pdmp3.comments_entry), "");
   gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(pdmp3.genre_combo)->entry), "");
}

/*
 * Convenience function to create an
 * ok/cancel dialog box
 */

gint pdmp3_question_dialog(gchar *message)
{
   gint reply = 0;
   static GtkWidget *dialog = NULL;
   GtkWidget *label;

   dialog = gnome_dialog_new(PACKAGE, GNOME_STOCK_BUTTON_YES, GNOME_STOCK_BUTTON_NO, NULL);
   
   gnome_dialog_set_close(GNOME_DIALOG(dialog), TRUE);
   gnome_dialog_close_hides(GNOME_DIALOG(dialog), TRUE);
   gnome_dialog_set_default(GNOME_DIALOG(dialog), GNOME_NO);
   
   label = gtk_label_new(message);
   
   gtk_box_pack_start(GTK_BOX(GNOME_DIALOG(dialog)->vbox),
		      label, TRUE, TRUE, GNOME_PAD);
   
   gtk_widget_show_all(dialog);
   
   reply = gnome_dialog_run(GNOME_DIALOG(dialog));
   
   gtk_widget_destroy(dialog);
   
   return(reply);
}

/*
 * Called whenever the main window is destroyed.
 * Saves our gui apperance and geometry
 */

void pdmp3_gui_shutdown(void)
{
   gint i;
   gint cols = GTK_CLIST(pdmp3.file_list)->columns;
   GtkCList *list = GTK_CLIST(pdmp3.file_list);
   GString *buf = g_string_new(NULL);
   gint x = 0;
   gint y = 0;
   gint width = 800;
   gint height = 600;
   
   gdk_window_get_position(pdmp3.main_window->window, &x, &y);
   gdk_window_get_size(pdmp3.main_window->window, &width, &height);
   
   gnome_config_set_int("/pdmp3/geometry/pdmp3_x", x);
   gnome_config_set_int("/pdmp3/geometry/pdmp3_y", y);
   gnome_config_set_int("/pdmp3/geometry/pdmp3_width", width);
   gnome_config_set_int("/pdmp3/geometry/pdmp3_height", height);
   gnome_config_set_int("/pdmp3/geometry/pdmp3_vpaned", pdmp3.file_list_scrolled_window->allocation.height);
   gnome_config_set_int("/pdmp3/geometry/pdmp3_hpaned", pdmp3.dir_browser_scrolled_window->allocation.width);

   for(i = 0; i < cols; i++)
     {
	g_string_sprintf(buf, "/pdmp3/file_list/column_%d", i);
	gnome_config_set_int(buf->str, (int)list->column[i].width);
     }
   g_string_free(buf, TRUE);
   gnome_config_sync();
}

/*
 * Convenience function to enable or disable
 * certain menu items depending on program state
 */

void pdmp3_update_menubar(void)
{
   if(pdmp3.selected_file)
     {
	gtk_widget_set_sensitive(file_menu[0].widget, TRUE);
	gtk_widget_set_sensitive(file_menu[1].widget, TRUE);
	gtk_widget_set_sensitive(file_menu[2].widget, TRUE);
	gtk_widget_set_sensitive(edit_menu[0].widget, TRUE);
	gtk_widget_set_sensitive(edit_menu[2].widget, TRUE);
     }
   else
     {
	gtk_widget_set_sensitive(file_menu[0].widget, FALSE);
	gtk_widget_set_sensitive(file_menu[1].widget, FALSE);
	gtk_widget_set_sensitive(file_menu[2].widget, FALSE);
	gtk_widget_set_sensitive(edit_menu[0].widget, FALSE);
	gtk_widget_set_sensitive(edit_menu[2].widget, FALSE);
     }
   if(pdmp3.selected_dir)
     {
	gtk_widget_set_sensitive(file_menu[4].widget, TRUE);
     }
   else
     {
	gtk_widget_set_sensitive(file_menu[4].widget, FALSE);
     }
}

void pdmp3_update_toolbar(void)
{
   if(pdmp3.selected_file)
     {
	gtk_widget_set_sensitive(toolbar[1].widget, TRUE);
	gtk_widget_set_sensitive(toolbar[4].widget, TRUE);
	gtk_widget_set_sensitive(toolbar[6].widget, TRUE);
     }
   else
     {
	gtk_widget_set_sensitive(toolbar[1].widget, FALSE);
	gtk_widget_set_sensitive(toolbar[4].widget, FALSE);
	gtk_widget_set_sensitive(toolbar[6].widget, FALSE);
     }
}

void pdmp3_lock_gui(void)
{
   gtk_widget_set_sensitive(GTK_WIDGET(pdmp3.dir_browser), FALSE);
   gtk_widget_set_sensitive(GTK_WIDGET(pdmp3.file_list), FALSE);
   gtk_widget_set_sensitive(GTK_WIDGET(pdmp3.artist_entry), FALSE);
   gtk_widget_set_sensitive(GTK_WIDGET(pdmp3.title_entry), FALSE);
   gtk_widget_set_sensitive(GTK_WIDGET(pdmp3.album_entry), FALSE);
   gtk_widget_set_sensitive(GTK_WIDGET(pdmp3.year_entry), FALSE);
   gtk_widget_set_sensitive(GTK_WIDGET(pdmp3.comments_entry), FALSE);
   gtk_widget_set_sensitive(GTK_WIDGET(pdmp3.genre_combo), FALSE);
   gtk_widget_set_sensitive(GTK_WIDGET(file_menu[0].widget), FALSE);
   gtk_widget_set_sensitive(GTK_WIDGET(file_menu[1].widget), FALSE);
   gtk_widget_set_sensitive(GTK_WIDGET(file_menu[2].widget), FALSE);
   gtk_widget_set_sensitive(GTK_WIDGET(file_menu[4].widget), FALSE);
   gtk_widget_set_sensitive(GTK_WIDGET(file_menu[5].widget), FALSE);
   gtk_widget_set_sensitive(GTK_WIDGET(edit_menu[0].widget), FALSE);
   gtk_widget_set_sensitive(GTK_WIDGET(edit_menu[1].widget), FALSE);
   gtk_widget_set_sensitive(GTK_WIDGET(edit_menu[2].widget), FALSE);
   gtk_widget_set_sensitive(GTK_WIDGET(toolbar[1].widget), FALSE);
   gtk_widget_set_sensitive(GTK_WIDGET(toolbar[2].widget), FALSE);
   gtk_widget_set_sensitive(GTK_WIDGET(toolbar[4].widget), FALSE);
   gtk_widget_set_sensitive(GTK_WIDGET(toolbar[5].widget), FALSE);
   gtk_widget_set_sensitive(GTK_WIDGET(toolbar[6].widget), FALSE);
}

void pdmp3_unlock_gui(void)
{
      gtk_widget_set_sensitive(GTK_WIDGET(pdmp3.dir_browser), TRUE);
      gtk_widget_set_sensitive(GTK_WIDGET(pdmp3.file_list), TRUE);
      gtk_widget_set_sensitive(GTK_WIDGET(pdmp3.artist_entry), TRUE);
      gtk_widget_set_sensitive(GTK_WIDGET(pdmp3.title_entry), TRUE);
      gtk_widget_set_sensitive(GTK_WIDGET(pdmp3.album_entry), TRUE);
      gtk_widget_set_sensitive(GTK_WIDGET(pdmp3.year_entry), TRUE);
      gtk_widget_set_sensitive(GTK_WIDGET(pdmp3.comments_entry), TRUE);
      gtk_widget_set_sensitive(GTK_WIDGET(pdmp3.genre_combo), TRUE);
      gtk_widget_set_sensitive(GTK_WIDGET(file_menu[0].widget), TRUE);
      gtk_widget_set_sensitive(GTK_WIDGET(file_menu[1].widget), TRUE);
      gtk_widget_set_sensitive(GTK_WIDGET(file_menu[2].widget), TRUE);
      gtk_widget_set_sensitive(GTK_WIDGET(file_menu[4].widget), TRUE);
      gtk_widget_set_sensitive(GTK_WIDGET(file_menu[5].widget), TRUE);
      gtk_widget_set_sensitive(GTK_WIDGET(edit_menu[0].widget), TRUE);
      gtk_widget_set_sensitive(GTK_WIDGET(edit_menu[1].widget), TRUE);
      gtk_widget_set_sensitive(GTK_WIDGET(edit_menu[2].widget), TRUE);
      gtk_widget_set_sensitive(GTK_WIDGET(toolbar[1].widget), TRUE);
      gtk_widget_set_sensitive(GTK_WIDGET(toolbar[2].widget), TRUE);
      gtk_widget_set_sensitive(GTK_WIDGET(toolbar[4].widget), TRUE);
      gtk_widget_set_sensitive(GTK_WIDGET(toolbar[5].widget), TRUE);
      gtk_widget_set_sensitive(GTK_WIDGET(toolbar[6].widget), TRUE);
}

/*
 * Create the entire gui
 */

void pdmp3_create(void)
{
   GtkWidget *label;
   GtkWidget *vbox;
   GtkWidget *tag_table;
   GtkWidget *frame;
   GtkTargetEntry target_table[] = {
	{ "text/plain", 0, 0 }
   };
   
   gchar *titles[] = { " ", "File Name", "File Size", "File Date", NULL };
   
   pdmp3.main_window = gnome_app_new(PACKAGE, PACKAGE);
   gtk_signal_connect(GTK_OBJECT(pdmp3.main_window), "delete_event",
		      GTK_SIGNAL_FUNC(pdmp3_quit_cb), NULL);
   
   /* If the geometry is specifed then use
    * that.  If not then use our defaults
    */
   if(pdmp3.geometry != NULL)
     {
	gint x, y, w, h;
	if(gnome_parse_geometry(pdmp3.geometry, &x, &y, &w, &h))
	  {
	     if(x != -1)
	       gtk_widget_set_uposition(GTK_WIDGET(pdmp3.main_window), x, y);
	     if(w != -1)
	       gtk_window_set_default_size(GTK_WINDOW(pdmp3.main_window), w, h);
	  }
	else
	  g_error(_("Unable to parse geometry string '%s'"), pdmp3.geometry);
     }
   else
     {
	gint x, y, w, h;
	
	w = gnome_config_get_int("/pdmp3/geometry/pdmp3_width");
	h = gnome_config_get_int("/pdmp3/geometry/pdmp3_height");
	
	if(w != 0 && h != 0)
	  gtk_window_set_default_size(GTK_WINDOW(pdmp3.main_window), w, h);
	else
	  gtk_window_set_default_size(GTK_WINDOW(pdmp3.main_window), 800, 600);
	x = gnome_config_get_int("/pdmp3/geometry/pdmp3_x");
	y = gnome_config_get_int("/pdmp3/geometry/pdmp3_y");
     }
   
   /* create the menubar */
   gnome_app_create_menus(GNOME_APP(pdmp3.main_window), menubar);
   
   /* create the toolbar */
   gnome_app_create_toolbar(GNOME_APP(pdmp3.main_window), toolbar);
   
   /* create the statusbar */
   pdmp3.statusbar = gnome_appbar_new(TRUE, TRUE, GNOME_PREFERENCES_USER);
   gnome_app_set_statusbar(GNOME_APP(pdmp3.main_window), pdmp3.statusbar);
   gnome_appbar_set_status(GNOME_APPBAR(pdmp3.statusbar), PACKAGE);
   pdmp3.progress_bar = gnome_appbar_get_progress(GNOME_APPBAR(pdmp3.statusbar));
   
   /* create our containers */
   vbox = gtk_vbox_new(FALSE, 0);
   pdmp3.hpane = gtk_hpaned_new();
   gtk_paned_set_position(GTK_PANED(pdmp3.hpane), gnome_config_get_int_with_default("/pdmp3/geometry/pdmp3_hpaned=193", NULL));
   pdmp3.vpane = gtk_vpaned_new();
   gtk_paned_set_position(GTK_PANED(pdmp3.vpane), gnome_config_get_int_with_default("/pdmp3/geometry/pdmp3_vpaned=250", NULL));
   tag_table = gtk_table_new(6, 4, FALSE);
   frame = gtk_frame_new("ID3 Tag");
   
   /* create the scrolled window for 
    * the directory browser */
   pdmp3.dir_browser_scrolled_window = gtk_scrolled_window_new(NULL, NULL);
   gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(pdmp3.dir_browser_scrolled_window),
				  GTK_POLICY_AUTOMATIC,
				  GTK_POLICY_ALWAYS);

   /* create the dir browser */
   pdmp3.dir_browser = gtk_dir_browser_new();
   gtk_container_add(GTK_CONTAINER(pdmp3.dir_browser_scrolled_window), pdmp3.dir_browser);
   gtk_dir_browser_open_dir(GTK_DIR_BROWSER(pdmp3.dir_browser), pdmp3.selected_dir);
   pdmp3.dir_browser_menu = gnome_popup_menu_new(dir_browser_menu_popup);
   gtk_drag_dest_set(pdmp3.dir_browser, GTK_DEST_DEFAULT_ALL,
		     target_table, 1,
		     GDK_ACTION_COPY);
   gtk_signal_connect(GTK_OBJECT(pdmp3.dir_browser), "select_row",
		      GTK_SIGNAL_FUNC(pdmp3_dir_browser_row_selected_cb), NULL);
   gtk_signal_connect(GTK_OBJECT(pdmp3.dir_browser), "button_press_event",
		      GTK_SIGNAL_FUNC(pdmp3_dir_browser_button_pressed_cb), NULL);
   gtk_signal_connect(GTK_OBJECT(pdmp3.dir_browser), "drag_drop",
		       GTK_SIGNAL_FUNC(pdmp3_dir_browser_drop_cb), NULL);
   gtk_signal_connect(GTK_OBJECT(pdmp3.dir_browser), "drag_motion",
		      GTK_SIGNAL_FUNC(pdmp3_drag_motion_cb), NULL);
   
   /* create the scrolled window 
    * for the file list */
   pdmp3.file_list_scrolled_window = gtk_scrolled_window_new(NULL, NULL);
   gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(pdmp3.file_list_scrolled_window),
				  GTK_POLICY_AUTOMATIC,
				  GTK_POLICY_ALWAYS);
   
   /* create the file list */
   pdmp3.file_list = gtk_clist_new_with_titles(4, titles);
   gtk_clist_set_selection_mode(GTK_CLIST(pdmp3.file_list), GTK_SELECTION_SINGLE);
   gtk_clist_set_shadow_type(GTK_CLIST(pdmp3.file_list), GTK_SHADOW_OUT);
   gtk_container_add(GTK_CONTAINER(pdmp3.file_list_scrolled_window), pdmp3.file_list);
   gtk_clist_column_titles_show(GTK_CLIST(pdmp3.file_list));
   gtk_clist_set_column_width(GTK_CLIST(pdmp3.file_list), 0, 16);
   gtk_clist_set_column_min_width(GTK_CLIST(pdmp3.file_list), 0, 16);
   gtk_clist_set_column_max_width(GTK_CLIST(pdmp3.file_list), 0, 16);
   gtk_clist_set_column_width(GTK_CLIST(pdmp3.file_list), 1, gnome_config_get_int_with_default("/pdmp3/file_list/column_1=300", NULL));
   gtk_clist_set_column_width(GTK_CLIST(pdmp3.file_list), 2, gnome_config_get_int_with_default("/pdmp3/file_list/column_2=115", NULL));
   gtk_clist_set_column_width(GTK_CLIST(pdmp3.file_list), 3, gnome_config_get_int_with_default("/pdmp3/file_list/column_3=115", NULL));
   pdmp3.file_list_menu = gnome_popup_menu_new(file_list_menu_popup);
   gtk_drag_source_set(pdmp3.file_list,
		       GDK_BUTTON1_MASK, target_table, 1, GDK_ACTION_COPY);
   gtk_signal_connect(GTK_OBJECT(pdmp3.file_list), "select_row",
		      GTK_SIGNAL_FUNC(pdmp3_file_list_row_selected_cb), NULL);
   gtk_signal_connect(GTK_OBJECT(pdmp3.file_list), "button_press_event",
		      GTK_SIGNAL_FUNC(pdmp3_file_list_button_pressed_cb), NULL);
   gtk_signal_connect(GTK_OBJECT(pdmp3.file_list), "drag-begin",
		      GTK_SIGNAL_FUNC(pdmp3_drag_begin_cb), NULL);
   gtk_signal_connect(GTK_OBJECT(pdmp3.file_list), "drag-end",
		      GTK_SIGNAL_FUNC(pdmp3_drag_end_cb), NULL);
   
   /* create the field entries 
    * and labels */
   label = gtk_label_new(_("Artist"));
   gtk_table_attach(GTK_TABLE(tag_table), label, 0, 1, 0, 1,
		    GTK_SHRINK | GTK_EXPAND,
		    GTK_SHRINK | GTK_EXPAND, 5, 5);
   pdmp3.artist_entry = gtk_entry_new_with_max_length(30);
   gtk_table_attach(GTK_TABLE(tag_table), pdmp3.artist_entry, 1, 4, 0, 1,
		    GTK_SHRINK | GTK_EXPAND | GTK_FILL, 
		    GTK_SHRINK | GTK_EXPAND | GTK_FILL, 5, 5);
   
   label = gtk_label_new(_("Song Title"));
   gtk_table_attach(GTK_TABLE(tag_table), label, 0, 1, 1, 2,
		    GTK_SHRINK | GTK_EXPAND,
		    GTK_SHRINK | GTK_EXPAND, 5, 5);
   pdmp3.title_entry = gtk_entry_new_with_max_length(30);
   gtk_table_attach(GTK_TABLE(tag_table), pdmp3.title_entry, 1, 4, 1, 2,
		    GTK_SHRINK | GTK_EXPAND | GTK_FILL,
		    GTK_SHRINK | GTK_EXPAND | GTK_FILL, 5, 5);
   
   label = gtk_label_new(_("Album Title"));
   gtk_table_attach(GTK_TABLE(tag_table), label, 0, 1, 2, 3,
		    GTK_SHRINK | GTK_EXPAND,
		    GTK_SHRINK | GTK_EXPAND, 5, 5);
   pdmp3.album_entry = gtk_entry_new_with_max_length(30);
   gtk_table_attach(GTK_TABLE(tag_table), pdmp3.album_entry, 1, 4, 2, 3,
		    GTK_SHRINK | GTK_EXPAND | GTK_FILL,
		    GTK_SHRINK | GTK_EXPAND | GTK_FILL, 5, 5);
   
   label = gtk_label_new(_("Year"));
   gtk_table_attach(GTK_TABLE(tag_table), label, 0, 1, 3, 4,
		    GTK_SHRINK | GTK_EXPAND,
		    GTK_SHRINK | GTK_EXPAND, 5, 5);
   pdmp3.year_entry = gtk_entry_new_with_max_length(4);
   gtk_table_attach(GTK_TABLE(tag_table), pdmp3.year_entry, 1, 4, 3, 4,
		    GTK_SHRINK | GTK_EXPAND | GTK_FILL,
		    GTK_SHRINK | GTK_EXPAND | GTK_FILL, 5, 5);
   
   label = gtk_label_new(_("Comments"));
   gtk_table_attach(GTK_TABLE(tag_table), label, 0, 1, 4, 5,
		    GTK_SHRINK | GTK_EXPAND,
		    GTK_SHRINK | GTK_EXPAND, 5, 5);
   pdmp3.comments_entry = gtk_entry_new_with_max_length(30);
   gtk_table_attach(GTK_TABLE(tag_table), pdmp3.comments_entry, 1, 4, 4, 5,
		    GTK_SHRINK | GTK_EXPAND | GTK_FILL,
		    GTK_SHRINK | GTK_EXPAND | GTK_FILL, 5, 5);
   
   label = gtk_label_new(_("Genre"));
   gtk_table_attach(GTK_TABLE(tag_table), label, 0, 1, 5, 6,
		    GTK_SHRINK | GTK_EXPAND,
		    GTK_SHRINK | GTK_EXPAND, 5, 5);
   pdmp3.genre_combo = gtk_combo_new();
   gtk_combo_set_popdown_strings(GTK_COMBO(pdmp3.genre_combo), pdmp3.genre_list);
   gtk_entry_set_editable(GTK_ENTRY(GTK_COMBO(pdmp3.genre_combo)->entry), FALSE);
   gtk_table_attach(GTK_TABLE(tag_table), pdmp3.genre_combo, 1, 4, 5, 6,
		    GTK_SHRINK | GTK_EXPAND | GTK_FILL,
		    GTK_SHRINK | GTK_EXPAND | GTK_FILL, 5, 5);
   
   gtk_container_add(GTK_CONTAINER(frame), tag_table);

   /* add our widgets to our panes */
   gtk_paned_add1(GTK_PANED(pdmp3.vpane), pdmp3.file_list_scrolled_window);
   gtk_paned_add2(GTK_PANED(pdmp3.vpane), frame);
   
   gtk_paned_add1(GTK_PANED(pdmp3.hpane), pdmp3.dir_browser_scrolled_window);
   gtk_paned_add2(GTK_PANED(pdmp3.hpane), pdmp3.vpane);
   
   gtk_box_pack_end(GTK_BOX(vbox), pdmp3.hpane, TRUE, TRUE, 0);
   
   gnome_app_set_contents(GNOME_APP(pdmp3.main_window), vbox);

   pdmp3_load_file_list();
   
   pdmp3_update_menubar();
   pdmp3_update_toolbar();
   
   gtk_widget_show_all(pdmp3.main_window);
}
