/*****************************************************************************
 * PDMP3 - A MP3 manager for GNOME                                           *
 * Copyright (C) 2000 Scott Guilbeaux <scottg@viptx.net>                     *
 *                                                                           *
 * This program is free software; you can redistribute it and/or modify      *
 * it under the terms of the GNU General Public License as published by      *
 * the Free Software Foundation; either version 2 of the License, or         *
 * (at your option) any later version.                                       *
 *                                                                           *
 * This program is distributed in the hope that it will be useful,           *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of            *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
 * GNU General Public License for more details.                              *
 *                                                                           *
 * You should have received a copy of the GNU General Public License         *
 * along with this program; if not, write to the Free Software               *
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA *
 *****************************************************************************/

#include "pdmp3.h"

/*
 * Backup the contents of our tag so we
 * can use it in revert operations later
 */

void pdmp3_backup_tag(id3tag_t tag)
{
   FILE *fp;
   gchar *dir;
   gchar *path;
   gchar *file;
   int retval = 0;
   
   dir = gnome_util_prepend_user_home(".pdmp3");
   path = g_concat_dir_and_file(dir, g_basename(pdmp3.selected_file));
   file = g_strdup_printf("%s.tag", path);
   retval = g_file_exists(file);
   
   if(retval == 0)
     {
	fp = fopen(file, "w");
	if(fwrite((tag.artist), 1, 30, fp) < 30)
	  return;
	if(fwrite((tag.album), 1, 30, fp) < 30)
	  return;
	if(fwrite((tag.year), 1, 4, fp) < 4)
	  return;
	if(fwrite((tag.comment), 1, 30, fp) < 30)
	  return;
	if(fwrite(&(tag.genre), 1, 1, fp) < 1)
	  return;
	fclose(fp);
     }
}

/*
 * Read tag from selected file
 */

id3tag_t pdmp3_read_id3_tag(void)
{
   int retval = 0;
   id3tag_t tag;
   gchar *error;
   
   retval = id3_init(&tag);
   
   retval = id3_init_read_session(pdmp3.selected_file);
   
   if(retval)
     {
	tag = id3_read_tag();
	if(g_strncasecmp(tag.tag, "TAG", 3) == 0)
	  pdmp3_backup_tag(tag);
	id3_close_session();
     }
   else
     {
	pdmp3_clear_entries();
	error = g_strdup_printf(_("Could not open %s: %s"), pdmp3.selected_file, g_strerror(errno));
	gnome_error_dialog_parented(error, GTK_WINDOW(pdmp3.main_window));
     }
   return(tag);
}

/* 
 * Write the new id3 tag out to the file 
 */

void pdmp3_apply_tag(void)
{
   int retval = 0;
   id3tag_t tag;
   gchar *buffer;
   gchar *temp;
   unsigned char genre_id;
   
   if(pdmp3.selected_file)
     {
	/* get artist */
	buffer = gtk_entry_get_text(GTK_ENTRY(pdmp3.artist_entry));
	if(buffer[0] == '\0')
	  {
	     buffer[0] = ' ';
	     buffer[1] = '\0';
	  }
	strcpy(tag.artist, buffer);
	strcpy(tag.comment, (buffer[0] == '\0') ? " ": buffer);
	
	/* get song title */
	buffer = gtk_entry_get_text(GTK_ENTRY(pdmp3.title_entry));
	if(buffer[0] == '\0')
	  {
	     buffer[0] = ' ';
	     buffer[1] = '\0';
	  }
	strcpy(tag.title, buffer);
	strcpy(tag.comment, (buffer[0] == '\0') ? " ": buffer);
	
	/* get the album */
	buffer = gtk_entry_get_text(GTK_ENTRY(pdmp3.album_entry));
	if(buffer[0] == '\0')
	  {
	     buffer[0] = ' ';
	     buffer[1] = '\0';
	  }
	strcpy(tag.album, buffer);
	strcpy(tag.comment, (buffer[0] == '\0') ? " ": buffer);
	
	/* get the year */
	buffer = gtk_entry_get_text(GTK_ENTRY(pdmp3.year_entry));
	strcpy(tag.year, (buffer[0] == '\0') ? " ": buffer);

	/* get the comments */
	buffer = gtk_entry_get_text(GTK_ENTRY(pdmp3.comments_entry));
	strcpy(tag.comment, (buffer[0] == '\0') ? " ": buffer);
	
	/* get the genre */
	buffer = gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(pdmp3.genre_combo)->entry));
	if(strlen(buffer) == 0)
	  {
	     genre_id = 0xFF;
	  }
	else
	  genre_id = id3_get_genre_id_by_name(buffer);
	tag.genre = genre_id;
	
	/* let's write it out */
	retval = id3_init_write_session(pdmp3.selected_file);
	if(retval)
	  {
	     retval = id3_set_tag(tag);
	     if(retval)
	       {
		  id3_close_session();
	       }
	  }
	else
	  {
	     temp = g_strdup_printf(_("Could not open %s for writing: %s"), pdmp3.selected_file, g_strerror(errno));
	     gnome_error_dialog_parented(temp, GTK_WINDOW(pdmp3.main_window));
	  }
     }
}

/*
 * Revert back to the backup tag
 */

void pdmp3_revert_tag(void)
{
   FILE *fp;
   gchar *data_dir;
   gchar *temp;
   gchar *file;
   id3tag_t tag;
   int retval = 0;
   
   if(pdmp3.selected_file)
     {
	data_dir = gnome_util_prepend_user_home(".pdmp3");
	temp = g_concat_dir_and_file(data_dir, g_basename(pdmp3.selected_file));
	file = g_strdup_printf("%s.tag", temp);
	if(g_file_exists(file))
	  {
	     fp = fopen(file, "r");
	     if(fp)
	       {
		  strcpy(tag.tag, "TAG");
		  tag.tag[3] = '\0';
		  fread(&(tag.title), 1, 30, fp);
		  tag.title[30] = '\0';
		  fread(&(tag.artist), 1, 30, fp);
		  tag.artist[30] = '\0';
		  fread(&(tag.album), 1, 30, fp);
		  tag.album[30] = '\0';
		  fread(&(tag.year), 1, 4, fp);
		  tag.year[4] = '\0';
		  fread(&(tag.comment), 1, 30, fp);
		  tag.comment[30] = '\0';
		  fread(&(tag.genre), 1, 1, fp);
		  id3_field_trim(tag.title);
		  id3_field_trim(tag.artist);
		  id3_field_trim(tag.album);
		  id3_field_trim(tag.year);
		  id3_field_trim(tag.comment);
		  fclose(fp);
		  retval = id3_init_write_session(pdmp3.selected_file);
		  if(retval)
		    {
		       retval = id3_set_tag(tag);
		       if(retval)
			 {
			    pdmp3_display_id3_tag(tag);
			    id3_close_session();
			 }
		    }
		  else
		    {
		       temp = g_strdup_printf(_("Could not open %s for writing: %s"), pdmp3.selected_file, g_strerror(errno));
		       gnome_error_dialog_parented(temp, GTK_WINDOW(pdmp3.main_window));
		    }
	       }
	     else
	       {
		  temp = g_strdup_printf(_("Could not open %s for reading: %s"), file, g_strerror(errno));
		  gnome_error_dialog_parented(temp, GTK_WINDOW(pdmp3.main_window));
	       }
	  }
     }
}

/*
 * Deletes the selected file
 */

void pdmp3_delete_file(void)
{
   gint retval = 0;
   gchar *temp;
   
   retval = unlink(pdmp3.selected_file);
   
   if(retval < 0)
     {
	temp = g_strdup_printf(_("Could no delete %s: %s"), pdmp3.selected_file, g_strerror(errno));
	gnome_error_dialog_parented(temp, GTK_WINDOW(pdmp3.main_window));
     }
}
