/*****************************************************************************
 * PDMP3 - A MP3 manager for GNOME                                           *
 * Copyright (C) 2000 Scott Guilbeaux <scottg@viptx.net>                     *
 *                                                                           *
 * This program is free software; you can redistribute it and/or modify      *
 * it under the terms of the GNU General Public License as published by      *
 * the Free Software Foundation; either version 2 of the License, or         *
 * (at your option) any later version.                                       *
 *                                                                           *
 * This program is distributed in the hope that it will be useful,           *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of            *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
 * GNU General Public License for more details.                              *
 *                                                                           *
 * You should have received a copy of the GNU General Public License         *
 * along with this program; if not, write to the Free Software               *
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA *
 *****************************************************************************/

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef STDC_HEADERS
#include <stdio.h>
#include <stdlib.h>
#endif

#ifdef HAVE_UNISTD_H
#include <unistd.h>
#endif

#ifdef HAVE_STRING_H
#include <string.h>
#endif

#ifdef HAVE_FCNTL_H
#include <fcntl.h>
#endif

#ifdef HAVE_DIRENT_H
#include <dirent.h>
#endif

#ifdef HAVE_ERRNO_H
#include <errno.h>
#endif

#ifdef HAVE_SYS_TYPES_H
#include <sys/types.h>
#endif

#ifdef HAVE_SYS_STAT_H
#include <sys/stat.h>
#endif

#ifdef HAVE_SYS_DIR_H
#include <sys/dir.h>
#endif

#ifdef HAVE_SYS_MMAN_H
#include <sys/mman.h>
#endif

#include <pthread.h>

#include <gnome.h>

#include "id3.h"

#include "gtkdirbrowser.h"

#define COPYRIGHT "Copyright 2000 Scott Guilbeaux"
#define DESCRIPTION "PDMP3:  A MP3 manager for GNOME.  Please report bugs to the above email address."
#define HOMEPAGE "http://maul.viptx.net/software/pdmp3.html"

/* Struct to define the properties dialog */

typedef struct 
{
   GtkWidget *main_window;
   GtkWidget *apply_confirm_check_button;
   GtkWidget *revert_confirm_check_button;
   GtkWidget *delete_confirm_check_button;
   GtkWidget *override_start_dir_check_button;
   GtkWidget *move_confirm_check_button;
   GtkWidget *start_dir_entry;
   GtkWidget *start_dir_label;
   GtkWidget *auto_rename_recurse_check_button;
   GtkWidget *auto_rename_confirm_check_button;
   gint page_num;
} pdmp3_prop_dlg_t;

pdmp3_prop_dlg_t pdm3_prop_dlg;

/* Stuct to define the program prefs */

typedef struct 
{
   gchar *starting_dir;
   gint override_lastdir;
   gint apply_confirm;
   gint revert_confirm;
   gint delete_confirm;
   gint move_confirm;
   gint auto_rename_recurse;
   gint auto_rename_confirm;
} pdmp3_preferences_t;

/* Struct to define the rename dialog */
typedef struct
{
   GtkWidget *main_window;
   GtkWidget *file_entry;
} pdmp3_rename_dialog_t;

pdmp3_rename_dialog_t pdmp3_rename_dlg;

/* Struct to define the whole pdmp3 "object" */

typedef struct 
{
   GtkWidget *main_window;
   GtkWidget *statusbar;
   GtkWidget *vpane;
   GtkWidget *hpane;
   GtkWidget *dir_browser_scrolled_window;
   GtkWidget *dir_browser;
   GtkWidget *file_list_scrolled_window;
   GtkWidget *file_list;
   GtkWidget *artist_entry;
   GtkWidget *title_entry;
   GtkWidget *album_entry;
   GtkWidget *year_entry;
   GtkWidget *comments_entry;
   GtkWidget *genre_combo;
   GtkWidget *file_list_menu;
   GtkWidget *dir_browser_menu;
   GtkProgress *progress_bar;
   pdmp3_prop_dlg_t prop_dlg;
   pdmp3_preferences_t prefs;
   pdmp3_rename_dialog_t rename_dlg;
   GList *genre_list;
   gchar *selected_dir;
   gchar *selected_file;
   gchar *starting_dir;
   gchar *geometry;
   gint selected_file_list_row;
   gint selected_dir_browser_row;
   gint in_drag;
} pdmp3_t;

pdmp3_t pdmp3;

/* Some global thread flags */
gint play_list_thread_running;
gint rename_thread_running;

/* Some global menu stuff */
GnomeUIInfo file_menu[9];
GnomeUIInfo edit_menu[5];
GnomeUIInfo file_list_file_menu[4];
GnomeUIInfo file_list_edit_menu[4];
GnomeUIInfo auto_rename_menu[3];
GnomeUIInfo toolbar[8];

/* interface.c prototypes */
void pdmp3_clear_entries(void);
void pdmp3_create(void);
gint pdmp3_question_dialog(gchar *);
GtkWidget *gnome_pixmap_button_new_with_label(gchar **, gchar *, int, int, int);
void pdmp3_gui_shutdown(void);
void pdmp3_update_menubar(void);
void pdmp3_update_toolbar(void);
void pdmp3_lock_gui(void);
void pdmp3_unlock_gui(void);

/* callbacks.c prototypes */
void pdmp3_quit_cb(GtkWidget *, GdkEvent *, gpointer);
void pdmp3_about_cb(GtkWidget *, GdkEvent *, gpointer);
gint pdmp3_save_session_cb(GnomeClient *, gint, GnomeSaveStyle, gint, GnomeInteractStyle, gint, gpointer);
void pdmp3_session_die_cb(GnomeClient *, gpointer);
void pdmp3_file_list_row_selected_cb(GtkWidget *, gint, gint, GdkEventButton *, gpointer);
void pdmp3_file_list_button_pressed_cb(GtkWidget *, GdkEventButton *);
void pdmp3_dir_browser_row_selected_cb(GtkWidget *, gint, gint, GdkEventButton *, gpointer);
void pdmp3_dir_browser_button_pressed_cb(GtkWidget *, GdkEventButton *);
void pdmp3_clear_cb(GtkWidget *, GdkEvent *, gpointer);
void pdmp3_apply_cb(GtkWidget *, GdkEvent *, gpointer);
void pdmp3_revert_cb(GtkWidget *, GdkEvent *, gpointer);
void pdmp3_delete_cb(GtkWidget *, GdkEvent *, gpointer);
void pdmp3_preferences_cb(GtkWidget *, GdkEvent *, gpointer);
void pdmp3_start_dir_check_button_cb(GtkWidget *, GdkEvent *, gpointer);
void pdmp3_apply_confirm_check_button_cb(GtkWidget *, GdkEvent *, gpointer);
void pdmp3_revert_confirm_check_button_cb(GtkWidget *, GdkEvent *, gpointer);
void pdmp3_delete_confirm_check_button_cb(GtkWidget *, GdkEvent *, gpointer);
void pdmp3_move_confirm_check_button_cb(GtkWidget *, GdkEvent *, gpointer);
void pdpm3_auto_rename_recurse_check_button_cb(GtkWidget *, GdkEvent *, gpointer);
void pdmp3_auto_rename_confirm_check_button_cb(GtkWidget *, GdkEvent *, gpointer);
void pdmp3_preferences_apply_cb(GnomePropertyBox *, gint, gpointer);
void pdmp3_rescan_cb(GtkWidget *, GdkEvent *, gpointer);
void pdmp3_rename_cb(GtkWidget *, GdkEvent *, gpointer);
void pdmp3_file_info_cb(GtkWidget *, GdkEvent *, gpointer);
void pdmp3_play_list_cb(GtkWidget *, GdkEvent *, gpointer);
void pdmp3_launch_cb(GtkWidget *, gpointer);
void pdmp3_homepage_url_cb(GtkWidget *, const char *);
void pdmp3_dir_browser_drop_cb(GtkWidget *, GdkDragContext *, gint, gint, GtkSelectionData *, guint, guint);
void pdmp3_drag_motion_cb(GtkWidget *, GdkDragContext *, gint, gint, guint, gpointer);
void pdmp3_drag_begin_cb(GtkWidget *, GdkDragContext *, gpointer);
void pdmp3_drag_end_cb(GtkWidget *, GdkDragContext *, gpointer);
void pdmp3_space_to_underscore_cb(GtkWidget *, gpointer data);
void pdmp3_underscore_to_space_cb(GtkWidget *, gpointer data);

/* filelist.c prototypes */
void pdmp3_load_file_list(void);
void pdmp3_update_row(gint);

/* mp3fileio.c prototypes */
id3tag_t pdmp3_read_id3_tag(void);
void pdmp3_backup_tag(id3tag_t);
void pdmp3_apply_tag(void);
void pdmp3_revert_tag(void);
void pdmp3_delete_file(void);

/* tagdisplay.c prototypes */
void pdmp3_display_id3_tag(id3tag_t);
void pdmp3_display_id3_genre(gint);

/* init.c prototypes */
void pdmp3_setup_data_dir(void);
void pdmp3_init(void);
void pdmp3_read_config_options(void);

/* shutdown.c prototypes */
void pdmp3_write_config_options(void);
void pdmp3_shutdown(void);
void pdmp3_clear_data_dir(void);

/* preferences.c prototypes */
void pdmp3_preferences_window_create(void);

/* rename.c prototypes */
void pdmp3_rename_dialog_new(void);
void pdmp3_rename_file(gchar *, gchar *);
void pdmp3_underscore_to_space(void);
void pdmp3_space_to_underscore(void);
void pdmp3_underscore_to_space(void);

/* mp3fileinfo.c prototypes */
void pdmp3_info_window_new(void);

/* playlist.c prototypes */
void pdmp3_play_list_dlg_new(void);

/* launch.c prototypes */
void pdmp3_launch_program(gchar *);
