/*****************************************************************************
7 * PDMP3 - A MP3 manager for GNOME                                          *
 * Copyright (C) 2000 Scott Guilbeaux <scottg@viptx.net>                     *
 *                                                                           *
 * This program is free software; you can redistribute it and/or modify      *
 * it under the terms of the GNU General Public License as published by      *
 * the Free Software Foundation; either version 2 of the License, or         *
 * (at your option) any later version.                                       *
 *                                                                           *
 * This program is distributed in the hope that it will be useful,           *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of            *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
 * GNU General Public License for more details.                              *
 *                                                                           *
 * You should have received a copy of the GNU General Public License         *
 * along with this program; if not, write to the Free Software               *
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA *
 *****************************************************************************/

#include "pdmp3.h"

static FILE *fp;
static GtkFileSelection *fsel;

/*
 * Create an xmms compatible playlist from the selected direcrory,
 * handles subdirs recursively
 */

static void create_play_list(gchar *directory)
{
   DIR *dir;
   struct dirent *dirent;
   struct stat sb;
   gchar *fullpath;
   const gchar *ext;
   
   dir = opendir(directory);
   while((dirent = readdir(dir)))
     {
	/* skip . and .. */
	if((g_strncasecmp(dirent->d_name, ".", 1) != 0) && (g_strncasecmp(dirent->d_name, ".", 2) != 0))
	  {
	     fullpath = g_strdup_printf("%s%s", directory, dirent->d_name);
	     stat(fullpath, &sb);
	     if(S_ISDIR(sb.st_mode))
	       {
		  fullpath = g_strdup_printf("%s/", fullpath);
		  create_play_list(fullpath);
	       }
	     else
	       {
		  ext = g_extension_pointer(dirent->d_name);
		  if((g_strncasecmp(ext, "mp3", 3) == 0))
		    fprintf(fp, "%s\n", fullpath);
	       }
	  }
     }
}

/*
 * Thread function so the playlist creation
 * runs in the background
 */

static void create_play_list_t(gchar *directory)
{
   play_list_thread_running = TRUE;
   create_play_list(directory);
   fclose(fp);
   play_list_thread_running = FALSE;
}

/*
 * Callback for when they chose to crate a list
 */

static void file_selection_button_ok(GtkWidget *widget, GdkEvent *event, gpointer data)
{
   gchar *filename;
   pthread_t thread;
   
   filename = g_strdup_printf("%s", gtk_file_selection_get_filename(fsel));
   
   if(strlen(g_basename(filename)) > 0)
      {
	 gtk_widget_destroy(GTK_WIDGET(fsel));
	 fp = fopen(filename, "w");   
	 if(fp == NULL)
	   {
	      gchar *error = g_strdup_printf(_("Could not open %s: %s"), filename, g_strerror(errno));
	      gnome_error_dialog_parented(error, GTK_WINDOW(pdmp3.main_window));
	      return;
	   }
	 pthread_create(&thread, NULL, (void *)create_play_list_t, pdmp3.selected_dir);
	 pthread_detach(thread);
      }
   else
     gtk_widget_destroy(GTK_WIDGET(fsel));
}

/*
 * Create the playlist files selection dialog
 */

void pdmp3_play_list_dlg_new(void)
{
   fsel = GTK_FILE_SELECTION
     (gtk_file_selection_new (_("Save play list as")));
   gtk_signal_connect (GTK_OBJECT (fsel->ok_button), "clicked",
		       GTK_SIGNAL_FUNC (file_selection_button_ok), fsel);

   gtk_signal_connect_object
     (GTK_OBJECT (fsel->cancel_button), "clicked",
      GTK_SIGNAL_FUNC (gtk_widget_destroy), 
      GTK_OBJECT(fsel));
   
   gtk_window_position (GTK_WINDOW (fsel), GTK_WIN_POS_MOUSE);
   gtk_widget_show (GTK_WIDGET (fsel));
}
