/*****************************************************************************
 * PDMP3 - An MP3 manager for GNOME                                          *
 * Copyright (C) 2000 Scott Guilbeaux <scottg@viptx.net>                     *
 *                                                                           *
 * This program is free software; you can redistribute it and/or modify      *
 * it under the terms of the GNU General Public License as published by      *
 * the Free Software Foundation; either version 2 of the License, or         *
 * (at your option) any later version.                                       *
 *                                                                           *
 * This program is distributed in the hope that it will be useful,           *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of            *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
 * GNU General Public License for more details.                              *
 *                                                                           *
 * You should have received a copy of the GNU General Public License         *
 * along with this program; if not, write to the Free Software               *
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA *
 *****************************************************************************/

#include "pdmp3.h"

void pdmp3_preferences_window_create(void)
{
   gint x = 0;
   gint y = 0;
   GtkWidget *confirmations_frame;
   GtkWidget *confirmations_table;
   GtkWidget *environment_frame;
   GtkWidget *environment_table;
   GtkWidget *auto_rename_frame;
   GtkWidget *auto_rename_table;
   
   gdk_window_get_position(pdmp3.main_window->window, &x, &y);
   
   pdmp3.prop_dlg.main_window = gnome_property_box_new();
   gtk_widget_set_uposition(GTK_WIDGET(pdmp3.prop_dlg.main_window), x, y);
   
   confirmations_frame = gtk_frame_new(_("Confirmation Options"));
   confirmations_table = gtk_table_new(4, 1, FALSE);
   gtk_container_add(GTK_CONTAINER(confirmations_frame), confirmations_table);
   
   /* Confirm apply */
   pdmp3.prop_dlg.apply_confirm_check_button = gtk_check_button_new_with_label(
							 (_("Ask before applying the ID3 tag")));
   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(pdmp3.prop_dlg.apply_confirm_check_button),
				pdmp3.prefs.apply_confirm ? 1 : 0);
   gtk_table_attach(GTK_TABLE(confirmations_table), pdmp3.prop_dlg.apply_confirm_check_button,
		    0, 1, 0, 1, GTK_FILL | GTK_EXPAND | GTK_SHRINK,
		    GTK_FILL | GTK_EXPAND | GTK_SHRINK, 5, 5);
   gtk_signal_connect(GTK_OBJECT(pdmp3.prop_dlg.apply_confirm_check_button), 
		      "toggled", GTK_SIGNAL_FUNC(pdmp3_apply_confirm_check_button_cb), NULL);
   
   /* Confirm revert */
   pdmp3.prop_dlg.revert_confirm_check_button = gtk_check_button_new_with_label(
							  (_("Ask before reverting back to the original ID3 tag")));
   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(pdmp3.prop_dlg.revert_confirm_check_button),
				pdmp3.prefs.revert_confirm ? 1 : 0);
   gtk_table_attach(GTK_TABLE(confirmations_table), pdmp3.prop_dlg.revert_confirm_check_button,
		    0, 1, 1, 2, GTK_FILL | GTK_EXPAND | GTK_SHRINK,
		    GTK_FILL | GTK_EXPAND | GTK_SHRINK, 5, 5);
   gtk_signal_connect(GTK_OBJECT(pdmp3.prop_dlg.revert_confirm_check_button),
		      "toggled", GTK_SIGNAL_FUNC(pdmp3_apply_confirm_check_button_cb), NULL);
   
   /* Confirm delete */
   pdmp3.prop_dlg.delete_confirm_check_button = gtk_check_button_new_with_label(
							  (_("Ask before deleting the selected file")));
   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(pdmp3.prop_dlg.delete_confirm_check_button),
						  pdmp3.prefs.delete_confirm ? 1 : 0);
   gtk_table_attach(GTK_TABLE(confirmations_table), pdmp3.prop_dlg.delete_confirm_check_button,
		    0, 1, 2, 3, GTK_FILL | GTK_EXPAND | GTK_SHRINK,
		    GTK_FILL | GTK_EXPAND | GTK_SHRINK, 5, 5);
   gtk_signal_connect(GTK_OBJECT(pdmp3.prop_dlg.delete_confirm_check_button),
		      "toggled", GTK_SIGNAL_FUNC(pdmp3_delete_confirm_check_button_cb), NULL);
   
   /* Confirm move */
   pdmp3.prop_dlg.move_confirm_check_button = gtk_check_button_new_with_label(
							(_("Ask before moving the selected file")));
   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(pdmp3.prop_dlg.move_confirm_check_button),
				pdmp3.prefs.move_confirm ? 1 : 0);
   gtk_table_attach(GTK_TABLE(confirmations_table), pdmp3.prop_dlg.move_confirm_check_button,
		    0, 1, 3, 4, GTK_FILL | GTK_EXPAND | GTK_SHRINK,
		    GTK_FILL | GTK_EXPAND | GTK_SHRINK, 5, 5);
   gtk_signal_connect(GTK_OBJECT(pdmp3.prop_dlg.move_confirm_check_button),
		      "toggled", GTK_SIGNAL_FUNC(pdmp3_move_confirm_check_button_cb), NULL);
   
   gnome_property_box_append_page(GNOME_PROPERTY_BOX(pdmp3.prop_dlg.main_window),
				  confirmations_frame,
				  gtk_label_new("Confirmations"));
   
    /* ENVIRONMENT PAGE */

   environment_frame = gtk_frame_new("Environment Options");
   environment_table = gtk_table_new(2, 2, FALSE);
   
   /* Starting directory overrides */
   pdmp3.prop_dlg.override_start_dir_check_button = gtk_check_button_new_with_label(
							      (_("Override last working directory?")));
   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(pdmp3.prop_dlg.override_start_dir_check_button),
				pdmp3.prefs.override_lastdir ? 1 : 0);
   gtk_table_attach(GTK_TABLE(environment_table), pdmp3.prop_dlg.override_start_dir_check_button,
		    0, 1, 0, 1, GTK_FILL | GTK_EXPAND | GTK_SHRINK,
		    GTK_FILL | GTK_EXPAND | GTK_SHRINK, 5, 5);
   gtk_signal_connect(GTK_OBJECT(pdmp3.prop_dlg.override_start_dir_check_button),
		      "toggled", GTK_SIGNAL_FUNC(pdmp3_start_dir_check_button_cb), NULL);
   
   pdmp3.prop_dlg.start_dir_label = gtk_label_new(_("Starting directory:"));
   gtk_table_attach(GTK_TABLE(environment_table), pdmp3.prop_dlg.start_dir_label, 0, 1, 1, 2,
		    GTK_FILL | GTK_EXPAND | GTK_SHRINK,
		    GTK_FILL | GTK_EXPAND | GTK_SHRINK, 5, 5);
   
   pdmp3.prop_dlg.start_dir_entry = gtk_entry_new();
   gtk_entry_set_text(GTK_ENTRY(pdmp3.prop_dlg.start_dir_entry), pdmp3.prefs.starting_dir);
   gtk_table_attach(GTK_TABLE(environment_table), pdmp3.prop_dlg.start_dir_entry,
		    1, 2, 1, 2, GTK_FILL | GTK_EXPAND | GTK_SHRINK,
		    GTK_FILL | GTK_EXPAND | GTK_SHRINK, 5, 5);
   if(!pdmp3.prefs.override_lastdir)
     {
	gtk_widget_set_sensitive(pdmp3.prop_dlg.start_dir_label, FALSE);
	gtk_widget_set_sensitive(pdmp3.prop_dlg.start_dir_entry, FALSE);
     }
   else if(pdmp3.prefs.override_lastdir)
     {
	gtk_widget_set_sensitive(pdmp3.prop_dlg.start_dir_label, TRUE);
	gtk_widget_set_sensitive(pdmp3.prop_dlg.start_dir_entry, TRUE);
     }
   
   gtk_container_add(GTK_CONTAINER(environment_frame), environment_table);
   gnome_property_box_append_page(GNOME_PROPERTY_BOX(pdmp3.prop_dlg.main_window),
				  environment_frame,
				  gtk_label_new(_("Environment")));

   /* AUTO RENAME PAGE */
   auto_rename_frame = gtk_frame_new(_("Auto Rename Options"));
   auto_rename_table = gtk_table_new(2,1,FALSE);
   
   pdmp3.prop_dlg.auto_rename_confirm_check_button = gtk_check_button_new_with_label(
							       (_("Ask before auto renaming")));
   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(pdmp3.prop_dlg.auto_rename_confirm_check_button),
				pdmp3.prefs.auto_rename_confirm ? 1 : 0);
   gtk_signal_connect(GTK_OBJECT(pdmp3.prop_dlg.auto_rename_confirm_check_button),
		      "toggled", GTK_SIGNAL_FUNC(pdmp3_auto_rename_confirm_check_button_cb), NULL);
   gtk_table_attach(GTK_TABLE(auto_rename_table), pdmp3.prop_dlg.auto_rename_confirm_check_button,
		    0,1,0,1, GTK_FILL | GTK_EXPAND | GTK_SHRINK,
		    GTK_FILL | GTK_EXPAND | GTK_SHRINK, 5, 5);
   
   pdmp3.prop_dlg.auto_rename_recurse_check_button = gtk_check_button_new_with_label(
							       (_("Recursively enter sub directories when auto renaming")));
   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(pdmp3.prop_dlg.auto_rename_recurse_check_button),
				pdmp3.prefs.auto_rename_recurse ? 1 : 0);
   gtk_signal_connect(GTK_OBJECT(pdmp3.prop_dlg.auto_rename_recurse_check_button),
		      "toggled", GTK_SIGNAL_FUNC(pdpm3_auto_rename_recurse_check_button_cb), NULL);
   gtk_table_attach(GTK_TABLE(auto_rename_table), pdmp3.prop_dlg.auto_rename_recurse_check_button,
		    0, 1, 1, 2, GTK_FILL | GTK_EXPAND | GTK_SHRINK,
		    GTK_FILL | GTK_EXPAND | GTK_SHRINK, 5, 5);
   
   gtk_container_add(GTK_CONTAINER(auto_rename_frame), auto_rename_table);
   gnome_property_box_append_page(GNOME_PROPERTY_BOX(pdmp3.prop_dlg.main_window),
				  auto_rename_frame,
				  gtk_label_new(_("Auto Rename")));
   
   /* Setup our callbacks */
   gtk_signal_connect(GTK_OBJECT(pdmp3.prop_dlg.main_window), "apply",
		      GTK_SIGNAL_FUNC(pdmp3_preferences_apply_cb), NULL);
   gtk_signal_connect(GTK_OBJECT(pdmp3.prop_dlg.main_window), "help",
		      GTK_SIGNAL_FUNC(pdmp3_about_cb), NULL);
   
   gtk_widget_show_all(pdmp3.prop_dlg.main_window);
}
