/*****************************************************************************
 * PDMP3 - A MP3 manager for GNOME                                           *
 * Copyright (C) 2000 Scott Guilbeaux <scottg@viptx.net>                     *
 *                                                                           *
 * This program is free software; you can redistribute it and/or modify      *
 * it under the terms of the GNU General Public License as published by      *
 * the Free Software Foundation; either version 2 of the License, or         *
 * (at your option) any later version.                                       *
 *                                                                           *
 * This program is distributed in the hope that it will be useful,           *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of            *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
 * GNU General Public License for more details.                              *
 *                                                                           *
 * You should have received a copy of the GNU General Public License         *
 * along with this program; if not, write to the Free Software               *
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA *
 *****************************************************************************/

#include "pdmp3.h"

#define SPACE2UNDERSCORE 0
#define UNDERSCORE2SPACE 1

static GList *rename_list;
static gint rename_count;

void pdmp3_rename_file(gchar *src, gchar *dest)
{
   gint retval = 0;
   retval = rename(src, dest);
   if(retval < 0)
     {
	gchar *error;
	error = g_strdup_printf(_("Error renaming file: %s"), g_strerror(errno));
	gnome_error_dialog_parented(error, GTK_WINDOW(pdmp3.main_window));
	return;
     }
}

void pdmp3_rename_dialog_new(void)
{
   GtkWidget *frame;
   gint result = 0;
   gchar *temp;
   gchar *dest;
   
   /* Create the dialog and set the parent */
   pdmp3.rename_dlg.main_window = gnome_dialog_new(N_("Rename"),
						   GNOME_STOCK_BUTTON_OK,
						   GNOME_STOCK_BUTTON_CANCEL,
						   NULL);
   
   gnome_dialog_set_parent(GNOME_DIALOG(pdmp3.rename_dlg.main_window),
			   GTK_WINDOW(pdmp3.main_window));
   
   /* Create a frame to make it purdy */
   temp = g_strdup_printf(N_("Rename \"%s\""), g_basename(pdmp3.selected_file));
   frame = gtk_frame_new(temp);
   
   pdmp3.rename_dlg.file_entry = gtk_entry_new();
   gtk_entry_set_text(GTK_ENTRY(pdmp3.rename_dlg.file_entry), g_basename(pdmp3.selected_file));
   gtk_container_add(GTK_CONTAINER(frame), pdmp3.rename_dlg.file_entry);
   
   gtk_box_pack_start(GTK_BOX(GNOME_DIALOG(pdmp3.rename_dlg.main_window)->vbox),frame,
		                          FALSE,FALSE,0);
   gtk_widget_show_all(frame);
   for(;;)
     {
	result = gnome_dialog_run(GNOME_DIALOG(pdmp3.rename_dlg.main_window));
	if(result == 0 || result == 2)
	  {
	     dest = g_strdup_printf("%s/%s", pdmp3.selected_dir,
				    gtk_entry_get_text(GTK_ENTRY(pdmp3.rename_dlg.file_entry)));
	     
	     if(strlen(g_basename(dest)) == 0)
	       gnome_error_dialog_parented(_("You must enter a file name!"), GTK_WINDOW(pdmp3.main_window));
	     else if(strcmp(g_basename(dest), g_basename(pdmp3.selected_file)) == 0)
	       {}
	     else if(g_file_exists(dest))
	       {
		  gnome_error_dialog_parented(_("File already exists!"), GTK_WINDOW(pdmp3.main_window));
	       }
	     else
	       {
		  pdmp3_rename_file(pdmp3.selected_file, dest);
		  pdmp3_load_file_list();
		  gtk_clist_select_row(GTK_CLIST(pdmp3.file_list), 0, 0);
	       }
	     gnome_dialog_close(GNOME_DIALOG(pdmp3.rename_dlg.main_window));
	     break;
	  }
	else if(result < 0)
	  break;
	else if(result == 1)
	  {
	     gnome_dialog_close(GNOME_DIALOG(pdmp3.rename_dlg.main_window));
	     break;
	  }
     }
}

static void create_autorename_list(gchar *directory, gint type)
{
   DIR *dir;
   struct dirent *dirent;
   struct stat sb;
   gchar *fullpath;
   const gchar *ext;
   
   dir = opendir(directory);
   if(!dir)
     {
	gchar *err;
	err = g_strdup_printf(_("Could not open %s: %s"), directory, g_strerror(errno));
	gnome_error_dialog_parented(err, GTK_WINDOW(pdmp3.main_window));
     }
   else
     {
	while((dirent = readdir(dir)))
	  {
	     /* skip . and .. */
	     if((g_strncasecmp(dirent->d_name, ".", 1) != 0) && (g_strncasecmp(dirent->d_name, "..", 2) != 0))
	       {
		  fullpath = g_strdup_printf("%s%s", directory, dirent->d_name);
		  stat(fullpath, &sb);
		  if(S_ISDIR(sb.st_mode))
		    {
		       if(pdmp3.prefs.auto_rename_recurse)
			 {
			    fullpath = g_strdup_printf("%s/", fullpath);
			    create_autorename_list(fullpath, type);
			 }
		    }
		  else
		    {
		       ext = g_extension_pointer(dirent->d_name);
		       if((g_strncasecmp(ext, "mp3", 3) == 0))
			 {
			    if(type == SPACE2UNDERSCORE)
			      {
				 if(strstr(dirent->d_name, " "))
				   {
				      rename_list = g_list_append(rename_list, fullpath);
				      rename_count++;
				   }
			      }
			    else if(type == UNDERSCORE2SPACE)
			      {
				 if(strstr(dirent->d_name, "_"))
				   {
				      rename_list = g_list_append(rename_list, fullpath);
				      rename_count++;
				   }
			      }
			 }
		    }
	       }
	  }
     }
}

/*
 * SPACE TO UNDERSCORE
 */

static gchar *create_spaceless_filename(char *filename)
{
   gint pos = 0;
   gint i = 0;
   gchar *temp;
   
   temp = g_strdup_printf("%s", filename);
   
   for(i = strlen(filename); filename[i] != '/'; i--)
     {}
   pos = i + 1;
   for(; pos < strlen(filename); pos++)
     {
	if(filename[pos] == ' ')
	  temp[pos] = '_';
     }
   return(temp);
}

static void pdmp3_space_to_underscore_thread(void)
{
   GList *li;
   gfloat increment, value;
   
   gdk_threads_enter();
   pdmp3_lock_gui();
   gdk_threads_leave();
   
   rename_list = NULL;
   rename_count = 0;
   create_autorename_list(pdmp3.selected_dir, SPACE2UNDERSCORE);
   increment = (gfloat)1/(rename_count);
   value = increment;
   if(value > 1 || value == 0)
     value = 1;
   
   for(li = rename_list; li!= NULL; li = g_list_next(li)) {
      gchar *source, *dest, *message;
      source = g_strdup_printf("%s", (gchar *)li->data);
      dest = create_spaceless_filename(source);
      message = g_strdup_printf(_("Renaming \"%s\"..."), source);
      gdk_threads_enter();
      gnome_appbar_set_status(GNOME_APPBAR(pdmp3.statusbar), message);
      gdk_threads_leave();
      pdmp3_rename_file(source, dest);
      gdk_threads_enter();
      gtk_progress_set_percentage(GTK_PROGRESS(pdmp3.progress_bar), value);
      gdk_threads_leave();
      value += increment;
      if(value > 1 || value == 0)
	value = 1;
   }
   rename_list = NULL;
   rename_count = 0;
  
   gdk_threads_enter();
   pdmp3_unlock_gui();
   pdmp3_load_file_list();
   if(pdmp3.selected_file)
     gnome_appbar_set_status(GNOME_APPBAR(pdmp3.statusbar), pdmp3.selected_file);
   else
     gnome_appbar_set_status(GNOME_APPBAR(pdmp3.statusbar), PACKAGE);
   gtk_progress_set_percentage(GTK_PROGRESS(pdmp3.progress_bar), 0);
   pdmp3_update_menubar();
   pdmp3_update_toolbar();
   gdk_threads_leave();
}

void pdmp3_space_to_underscore(void)
{
   pthread_t thread;
   
   pthread_create(&thread, NULL, (void *)pdmp3_space_to_underscore_thread, NULL);
   pthread_detach(thread);
}

/*
 * UNDERSCORE TO SPACE
 */
static gchar *create_underscoreless_filename(gchar *filename)
{
   gint pos = 0;
   gint i = 0;
   gchar *temp;
   
   temp = g_strdup_printf("%s", filename);
   
   for(i = strlen(filename); filename[i] != '/'; i--)
     ;
   pos = 1 + 1;
   for(; pos < strlen(filename); pos++)
     {
	if(filename[pos] == '_')
	  temp[pos] = ' ';
     }
   return(temp);
}

static void pdmp3_underscore_to_space_thread(void)
{
   GList *li;
   gfloat increment, value;
   
   gdk_threads_enter();
   pdmp3_lock_gui();
   gdk_threads_leave();

   rename_list = NULL;
   rename_count = 0;
   create_autorename_list(pdmp3.selected_dir, UNDERSCORE2SPACE);
   increment = (gfloat)1/(rename_count);
   value = increment;
   if(value > 1 || value == 0)
     value = 1;
   
   for(li = rename_list; li != NULL; li = g_list_next(li))
     {
	gchar *source, *dest, *message;
	source = g_strdup_printf("%s", (gchar *)li->data);
	dest = create_underscoreless_filename(source);
	message = g_strdup_printf(_("Renaming \"%s\"..."), source);
	gdk_threads_enter();
	gnome_appbar_set_status(GNOME_APPBAR(pdmp3.statusbar), message);
	gdk_threads_leave();
	pdmp3_rename_file(source, dest);
	gdk_threads_enter();
	gtk_progress_set_percentage(GTK_PROGRESS(pdmp3.progress_bar), value);
	gdk_threads_leave();
	value += increment;
	if(value > 1 || value == 0)
	  value = 1;
     }
   rename_list = NULL;
   rename_count = 0;
   
   gdk_threads_enter();
   pdmp3_unlock_gui();
   pdmp3_load_file_list();
   if(pdmp3.selected_file)
     gnome_appbar_set_status(GNOME_APPBAR(pdmp3.statusbar), pdmp3.selected_file);
   else
     gnome_appbar_set_status(GNOME_APPBAR(pdmp3.statusbar), PACKAGE);
   gtk_progress_set_percentage(GTK_PROGRESS(pdmp3.progress_bar), 0);
   pdmp3_update_menubar();
   pdmp3_update_toolbar();
   gdk_threads_leave();
}

void pdmp3_underscore_to_space(void)
{
   pthread_t thread;
  
   pthread_create(&thread, NULL, (void *)pdmp3_underscore_to_space_thread, NULL);
   pthread_detach(thread);
}
